/* Layout Parser JBV 2012-2013 */

var LayoutParser = function(){
	
	this.references = {};
	this.elements = [];
	this.taggedelements = {};
	this.elementTokens = [ "define", "include", "import", // Structure management
						   "if", "loop",  // Data tree path exploration 
						   "gridlayout", "text", "textinput", "image" , "container", "empty", "linearlistlayout", "scrollcontainer", "mapper", "verticalbar", "button", "slider", // Elements management
						   "menubar" // Extended Elements
						 ];
	this.init = false;
	this.elementParser = {};
	this.elementPost = {};
	
} 

LayoutParser.prototype.recordFunctions = function() {
	
	// Structure management
	this.elementParser[ "define" ] = this.parseDefine;
	this.elementParser[ "include" ] = this.parseInclude;
	this.elementParser[ "import" ] = this.parseImport;
	
	// If/Loop
	this.elementParser[ "loop" ] = this.parseLoop;
	
	// Elements management
	this.elementParser[ "gridlayout" ] = this.parseGridLayout;
	this.elementParser[ "text" ] = this.parseTextElement;
	this.elementParser[ "textinput" ] = this.parseTextInputElement;
	this.elementParser[ "image" ] = this.parseImageElement;
	this.elementParser[ "container" ] = this.parseContainerElement;
	this.elementParser[ "empty" ] = this.parseEmptyElement;
	this.elementParser[ "linearlistlayout" ] = this.parseLinearListLayout;
	this.elementParser[ "mapper" ] = this.parseMapperElement;
	this.elementParser[ "scrollcontainer" ] = this.parseScrollContainerElement;
	this.elementParser[ "verticalbar" ] = this.parseVerticalBarElement;
	this.elementParser[ "menubar" ] = this.parseMenuBarElement;
	this.elementParser[ "slider" ] = this.parseSliderElement;
	
	this.elementPost[ "textinput" ] = this.postTextInputElement;
	
}

LayoutParser.prototype.load = function(iLayoutURL,iDataTree) {

	var aData;
	$.ajaxSetup( { "async": false } );
	$.getJSON(iLayoutURL, function(data) {
		aData = data;
	});
	$.ajaxSetup( { "async": true } );

	if (valid(aData)) {
		return this.parseLayout(aData,iDataTree);
	}
	return;
}

LayoutParser.prototype.parseLayout = function(iData, iDataTree) {

	if (!this.init) {
		this.recordFunctions();
		this.init = true;
	}

	this.references = {};
	this.elements = [];
	this.taggedelements = {};
	
	var aLayout = this.parse(iData, iDataTree);
	this.postProcess(); // build links between objects
	
	aLayout.tag = this.taggedelements;
	
	return aLayout;
	
}

// This function go through the data tree i.e I will receive "node1.node2"
// So the position in the data tree will be 
LayoutParser.prototype.setPosition =  function(iSteps,iDataTree){

	//console.log("Steps:"+iSteps);
	//console.log(iSteps);
	var aSteps = iSteps.split(".");
	var aDataTree = iDataTree;
	//console.log("setPosition");
	for (var i=0; i<aSteps.length; i++) {
		//console.log(aDataTree);
		//console.log(aSteps[i]);
		
		var aStep = aSteps[i];
		var startBracket = aStep.search("\\[");
		var stopBracket = aStep.search("\\]");
		var haveIndex = false;
		var index;
		if ((startBracket!=-1) && (stopBracket!=-1)) {
			haveIndex = true;
			var extract = aStep.substring(startBracket+1,stopBracket);
			//console.log("Extract:"+extract);
			index = parseInt( extract, 10);
			
			aStep =  aStep.substring(0,startBracket);
			//console.log("New aStep:"+aStep);
		}		
		
		if (valid(aDataTree[aStep])) {
			
			aDataTree = aDataTree[aStep];
			if (haveIndex) {
			   aDataTree = aDataTree[index];
			} 
			
		} else {
			logError("LayoutParser : Error when parsing layout, can't reach the data "+iSteps+".", iDataTree);
			
			return;
		}
	}
	return aDataTree;
	
}

// Concat several data
LayoutParser.prototype.concat = function(iElement,iDataTree) {
	
	var aElements = [];
	for (var i=0; i<iElement.length; i++) { // TODO test if iElement is an array
		//console.log(iElement[i]);
		aElements.push(this.convertData(iElement[i],iDataTree));
	}
	return aElements.join("");
	
}

// This will shorten layout implementation
// Here strict implementation to get data from the tree use "." 
// Directly after json separator i.e  ".data" will search in data tree 
// ".node.data" will work also
// " .data" will be considered as direct data
// "\.data" will be considered as direct data ".data"
LayoutParser.prototype.convertData = function(iElement,iDataTree) {
	//console.log("convertData :"+iElement);
	if (valid(iElement.agreg)) { // ask to concat all data
	
		//console.log("agreg");
		return this.concat(iElement.agreg, iDataTree);
	
	} else if (valid(iElement.eval)) { // ask to eval string
	
		//console.log("evaluate");
		return eval(this.convertData(iElement.eval,iDataTree));
	
	} else if (valid(iElement.formatdate)) {
		
		//console.log("Formatting a date :");
		
		if (valid(iElement.formatdate.from) && valid(iElement.formatdate.to) && valid(iElement.formatdate.data)) {
		
			var aData = this.convertData(iElement.formatdate.data,iDataTree);
			var aDate = moment(aData,iElement.formatdate.from);
			//console.log(moment().format());
			return aDate.format(iElement.formatdate.to);
		
		} 
		
	} else { // Consider iElement has a string

		/*console.log("Element");
		console.log(iElement);
		console.log(typeof iElement);*/
	
		if ( typeof iElement === "string") {
			if (iElement.indexOf(".")==0) { // Consider this as data element
				var name = iElement.substring(1);
				return this.setPosition(name,iDataTree);
			} else if (iElement.indexOf("\\.")==0) { // Consider this as direct data
				return iElement.substring(1);
			} else {
				return iElement;
			}
		} else {
			return iElement;
		}
	
	}
}

LayoutParser.prototype.parse = function(iData,iDataTree) { // TODO Create a hashmap of independent parsers

	for (var i=0; i<this.elementTokens.length; i++) {
	
		var aElement = iData[this.elementTokens[i]];
		this.decodingFunction = this.elementParser[this.elementTokens[i]];
	
		if (valid(aElement) && valid(this.decodingFunction)) {
		
			var aDataTree = iDataTree;
			if (valid(aElement["use"])) { // Detect "use" term in the element, this will go through the data tree
				aDataTree = this.setPosition( aElement["use"] , iDataTree );
			}
		
			var aParsedElement = this.decodingFunction(aElement,aDataTree);
		
			if (valid(aElement.tag)) {
				//console.log("record Tag:"+aElement.tag);
				var aTag = this.convertData(aElement.tag,aDataTree);
				this.taggedelements[aTag] = {
					content : aElement,
					element : aParsedElement
				}
				aParsedElement.tag = aTag; 	
			}
			
			if (valid(aElement.attach)) {
				//console.log("record Tag:"+aElement.tag);
				aParsedElement.attached = this.convertData(aElement.attach,aDataTree);	
			}
		
			this.elements.push({ content :  aElement, element : aParsedElement });
		
			return aParsedElement;
		
		} 
	
	}
	
	logError("LayoutParser : Error when parsing layout, element not found .", iData);
	return;

}



// Define for defining references + main layout
// can be used for duplicated elements
LayoutParser.prototype.parseDefine = function(iData,iDataTree) {
	
	if (valid(iData.references)) {
		
		var aReferences = iData.references;
		for (var i=0; i<aReferences.length; i++) {
		
			var reference = aReferences[i];
			
			if (valid(reference.name) && valid(reference.content)) {
				
				this.references[reference.name] = reference.content;
				
			} else {
			
				logError("LayoutParser : Error no reference name or content defined  :",aReference);
				return;

			}
		
		}	
	
	} // no reference defined
	
	// at the end we have to parse the main layout
	if (valid(iData.layout)) {
	
		return this.parse(iData.layout,iDataTree);
	
	} else {
		logError("LayoutParser : Error no define layout defined  :",iData);
		return;
	}
	
}

// Include element ... will include a reference at that point
LayoutParser.prototype.parseInclude = function(iData,iDataTree) {

	if (valid(iData.name)) {
	
		if (valid(this.references[iData.name])) {
			return this.parse(this.references[iData.name],iDataTree);
		} else {
			logError("LayoutParser : Error no reference defined for name "+iData.name+" layout defined  :",iData);
			return;
		}
		
	} else {
		logError("LayoutParser : Error in include no name defined  :",iData);
		return;
	}
	
}

LayoutParser.prototype.parseImport = function(iData,iDataTree) {

	if (valid(iData.url)) {

		var url = iData.url;
	
		var aData;
		$.ajaxSetup( { "async": false } );
		$.getJSON(url, function(data) {
			aData = data;
		});
		$.ajaxSetup( { "async": true } );
	
		return this.parse(aData,iDataTree);
	
	} else {
		logError("LayoutParser : Error in import no url defined  :",iData);
		return;
	}

}

// Loop/If
LayoutParser.prototype.parseLoop = function(iData,iDataTree) {

	// First parse a linear list layout
	var aLayout = this.parseLinearListLayout(iData,iDataTree);

	if (valid(aLayout)) {
		// Then loop on data tree
		if (valid(iData.elements)) {
	
			var aList = this.convertData(iData.elements,iDataTree);
			aList = [].concat( aList );
			//console.log(aList);
		
			if (valid(iData.model)){
			
				for (var i=0; i<aList.length; i++) {
					aLayout.pushElement(this.parse(iData.model,aList[i]));
				}
			
			} else {
				logError("LayoutParser : Error in loop no model defined  :",iData);
				return;
			}
		
		} else {
			logError("LayoutParser : Error in loop no elements defined  :",iData);
			return;
		}
	} else {
		logError("LayoutParser : Error in loop layout not defined  :",iData);
		return;
	}
	
	return aLayout;

}


// GridLayout Parser
LayoutParser.prototype.parseGridLayout = function(iData,iDataTree) {
	
	if (valid(iData.dim)) {
	
		if ( (!valid(iData.dim[0])) || (!valid(iData.dim[1])) ) {
			logError("LayoutParser : Error dim not valid in gridlayout :",aColumn);
			return;
		}
	
		var aGridLayout = new GridLayout(this.convertData(iData.dim[0],iDataTree),this.convertData(iData.dim[1],iDataTree));
			
		if (valid(iData.properties)) {
			aGridLayout.setProperties(this.convertData(iData.properties,iDataTree));
		}
		
		var columns = iData.columns;
		if (valid(columns)){
			
			for (var i=0; i<columns.length; i++) {
				var aColumn = columns[i];
				if (valid(aColumn.x) && valid(aColumn.properties)) {
				  aGridLayout.addColumnProperties(this.convertData(aColumn.x,iDataTree),this.convertData(aColumn.properties,iDataTree));	
				} else {
				  logError("LayoutParser : Error no column x or properties defined in gridlayout :",aColumn);
				  return;
				}
			}
			
		}
		
		var cells = iData.cells;	
		if (valid(cells)){
			
			for (var i=0; i<cells.length; i++) {
			
				var aCell = cells[i];
				if (valid(aCell.pos) && valid(aCell.inner)) {
					
					if (valid(aCell.pos[0]) && valid(aCell.pos[1])) {
				
						var aElement = this.parse(aCell.inner,iDataTree);	
						aGridLayout.add(this.convertData(aCell.pos[0],iDataTree),this.convertData(aCell.pos[1],iDataTree),aElement);
					
					} else {
						logError("LayoutParser : Error no pos value defined for gridlayout :",aCell);
						return;
					}
				
				} else {
				  logError("LayoutParser : Error no pos element or (inner component) defined for gridlayout :",aCell);
				  return;
				}
			}
		
		} else {
			logError("LayoutParser : Error no cell defined for gridlayout :",iData);
			return;
		}
			
		return aGridLayout;
			
	} else {
		logError("LayoutParser : Error no width and height defined for gridlayout :",iData);
		return;
	}
	
}

// TextElement Parser
LayoutParser.prototype.parseTextElement = function(iData,iDataTree) {

	if (valid(iData.text)) {
	
		var aTextElement = new TextElement(this.convertData(iData.text,iDataTree));
	
		// fontsize
		if (valid(iData.fontsize)) {
			aTextElement.setFontSize(this.convertData(iData.fontsize,iDataTree));
		}
	
		// position
		if (valid(iData.position)) {
			aTextElement.setPosition(this.convertData(iData.position,iDataTree));
		}
	
		// vertical position
		if (valid(iData.verticalposition)) {
			aTextElement.setVerticalPosition(this.convertData(iData.verticalposition,iDataTree));
		}
	
		if (valid(iData.isurl)) {
			if (iData.isurl) {
				aTextElement.isUrl();
			}
		}
	
		// translate
		if (valid(iData.translate)) {
			aTextElement.translate(this.convertData(iData.translate,iDataTree));
		}
		
		// color
		if (valid(iData.color)) {
			aTextElement.setColor(this.convertData(iData.color,iDataTree));
		}
		
		// fontsize
		if (valid(iData.linkcolor)) {
			aTextElement.setLinkColor(this.convertData(iData.linkcolor,iDataTree));
		}
		
		// color
		if (valid(iData.style)) {
			aTextElement.setStyle(this.convertData(iData.style,iDataTree));
		}
	
		if (valid(iData.onMouseOverColor)) {
			aTextElement.setMouseOverColor(this.convertData(iData.onMouseOverColor,iDataTree));
		}
		
		if (valid(iData.onMouseOverBkColor)) {
			aTextElement.setMouseOverBkColor(this.convertData(iData.onMouseOverBkColor,iDataTree));
		}
	
		return aTextElement;
	
	} else {
		logError("LayoutParser : Error no text defined for text :",iData);
		return;
	}

}

// InputTextElement Parser
LayoutParser.prototype.parseTextInputElement = function(iData,iDataTree) {

	if (valid(iData.text) && valid(iData.id)) {
	
		var aTextInputElement = new TextInputElement(this.convertData(iData.text,iDataTree),this.convertData(iData.id,iDataTree));
	
		// fontsize
		if (valid(iData.fontsize)) {
			aTextInputElement.setFontSize(this.convertData(iData.fontsize,iDataTree));
		}
	
		// position
		if (valid(iData.position)) {
			aTextInputElement.setPosition(this.convertData(iData.position,iDataTree));
		}
	
		// translate
		if (valid(iData.translate)) {
			aTextInputElement.translate(this.convertData(iData.translate,iDataTree));
		}
		
		// color
		if (valid(iData.color)) {
			aTextInputElement.setColor(this.convertData(iData.color,iDataTree));
		}
		
		// color
		if (valid(iData.padding)) {
			aTextInputElement.setPadding(this.convertData(iData.padding,iDataTree));
		}
		
		// color description
		if (valid(iData.colordesc)) {
			aTextInputElement.setColorDescription(this.convertData(iData.colordesc,iDataTree));
		}
	
		// Autocomplete function
		if (valid(iData.autocomplete)) {
			
			var auto = iData.autocomplete;
			if (valid(auto.source) && valid(auto.select) && valid(auto.min)) {
			
				aTextInputElement.setAutoComplete(
					window.recordedFunction[auto.source], 
					window.recordedFunction[auto.select],
					auto.min,
					auto.options);
			
			}
			
		}
	
		return aTextInputElement;
	
	} else {
		logError("LayoutParser : Error no text or id defined for textinput :",iData);
		return;
	}

}

// ImageElement Parser
LayoutParser.prototype.parseImageElement = function(iData,iDataTree) {

	if (valid(iData.url)) {
		
		var newURL = this.convertData(iData.url,iDataTree);
		var aImage =  new ImageElement(newURL);
		
		if (valid(iData.properties)) {
			aImage.setProperties(iData.properties);
		}
		
		if (valid(iData.onclick)) {
			aImage.onClick(window.recordedFunction[iData.onclick]);
		}
		
		return aImage;
		
	} else {
		logError("LayoutParser : Error no url element defined for image :",iData);
		return;
	}

}

// ContainerElement Parser
LayoutParser.prototype.parseContainerElement = function(iData,iDataTree) {

	if (valid(iData.inner)) {
	
		var inner = this.parse(iData.inner,iDataTree);
	
		var aContainer = new ContainerElement(inner);
	
		if (valid(iData.properties)) {
			aContainer.setProperties(iData.properties);
		}
		
		if (valid(iData.onenterbkcolor)) {
			aContainer.onEnterBkColor(iData.onenterbkcolor);
		}
		
		if (valid(iData.onclick)) {
			aContainer.onClick(window.recordedFunction[iData.onclick]);
		}
	
		if (valid(iData.style)) {
			aContainer.addStyle(this.convertData(iData.style,iDataTree));
		}
	
		return aContainer;
	
	} else {
		logError("LayoutParser : Error no inner element defined for container :",iData);
		return;
	}

}

// EmptyElement Parser
LayoutParser.prototype.parseEmptyElement = function(iData,iDataTree) {

	return new EmptyElement;

}

// LinearListLayout Parser
LayoutParser.prototype.parseLinearListLayout = function(iData,iDataTree) {

	if (valid(iData.orientation) && valid(iData.mode)) {
	
		var aLayout = new LinearListLayout(this.convertData(iData.orientation,iDataTree),this.convertData(iData.mode,iDataTree));
		
		if (valid(iData.align)) {
			aLayout.setAlignment(this.convertData(iData.align,iDataTree));
			
			if (valid(iData.verticalalign)) {
				aLayout.setAlignment(this.convertData(iData.align,iDataTree),this.convertData(iData.verticalalign,iDataTree));
			}
		}
		
		if (valid(iData.cells)) {
			
			for (var i=0; i<iData.cells.length; i++) {
			
				if (valid(iData.cellsizes)) {
					aLayout.pushElement(this.parse(iData.cells[i],iDataTree),iData.cellsizes[i]);
				} else {
					aLayout.pushElement(this.parse(iData.cells[i],iDataTree));
				}
			
			}
			
		}

		return aLayout;
	
	} else {
	
		logError("LayoutParser : Error on linear list layout definition , missing orientation or mode definition.",iData);
		return;
		
	}
	
}

LayoutParser.prototype.parseMapperElement = function(iData,iDataTree) {

	if (valid(iData.id)) {
		
		var aMapper = new MapperElement(this.convertData(iData.id,iDataTree));
		
		if (valid(iData.properties)) {
			aMapper.setProperties(iData.properties);
		}
		
		return aMapper;
	
	} else {
	
		logError("LayoutParser : Error on mapper definition , missing id .",iData);
		return;
	
	}

}

LayoutParser.prototype.parseScrollContainerElement = function(iData,iDataTree) {
	
	if (valid(iData.inner)) {
		
		var aScroll =  new ScrollContainerElement(this.parse(iData.inner,iDataTree));
	
		if (valid(iData.style)){
			aScroll.setStyle(iData.style);
		}
		
		if (valid(iData.canvasstyle)){
			aScroll.setCanvasStyle(iData.canvasstyle);
		}
	
		return aScroll;
	
	} else {

		logError("LayoutParser : Error on scroll container definition , missing id .",iData);
		return;
	
	}

}

LayoutParser.prototype.parseVerticalBarElement = function(iData,iDataTree) {

	var VBE = new VerticalBarElement();
	
	if (valid(iData.position)) {
		VBE.setPosition(iData.position);
	}
	
	if (valid(iData.style)) {
		VBE.setStyle(iData.style);
	}
	
	if (valid(iData.vstyle)) {
		VBE.setVStyle(iData.vstyle);
	}
	
	if (valid(iData.color)) {
		VBE.setColor(iData.color);
	}
	
	if (valid(iData.width)) {
		VBE.setWidth(iData.width);
	}

	if (valid(iData.height)) {
		VBE.setHeight(iData.height);
	}

	return VBE;

}

LayoutParser.prototype.parseMenuBarElement = function(iData,iDataTree) {

	var MB = new MenuBarElement();
	
	if (valid(iData.align)) {
		MB.setAlign(iData.align);
	}
	if (valid(iData.textcolor)) {
		MB.setTextColor(iData.textcolor);
	}
	if (valid(iData.fontsize)) {
		MB.setFontSize(iData.fontsize);
	}
	if (valid(iData.mouseovercolor)) {
		MB.setMouseOverColor(iData.mouseovercolor);
	}
	if (valid(iData.mouseoverbkcolor)) {
		MB.setMouseOverBkColor(iData.mouseoverbkcolor);
	}
	if (valid(iData.padding)) {
		MB.setPadding(iData.padding);
	}
	
	if (valid(iData.vstyle)) {
		MB.setVerticalBarStyle(iData.vstyle);
	}
	
	if (valid(iData.vcolor)) {
		MB.setVerticalBarColor(iData.vcolor);
	}
	
	if (valid(iData.vwidth)) {
		MB.setVerticalBarWidth(iData.vwidth);
	}
	
	if (valid(iData.vheight)) {
		MB.setVerticalBarHeight(iData.vheight);
	}

	if (valid(iData.elements)) {
		//console.log("Push elements in menu bar");
		MB.pushElements(iData.elements,iData.translates);
	}
	
	return MB;

}

LayoutParser.prototype.parseSliderElement = function(iData,iDataTree) {

	if (valid(iData.value) && valid(iData.range) && valid(iData.callback)) {
		
		var slider = new SliderElement(iData.value,iData.range,window.recordedFunction[iData.callback]);
		
		if (valid(iData.size)) {
			if (valid(iData.size.width) && valid(iData.size.height)) {
				slider.setSize(iData.size.width,iData.size.height);
			}
		}
		
		if (valid(iData.rangetype)) {
			slider.setRange(iData.range);
		}
			
		if (valid(iData.backgroundcolor)) {
			slider.setBackgroundColor(iData.backgroundcolor);
		}	
			
		return slider;

	} else {
	
		logError("LayoutParser : Error on slider definition , missing value, range, callback .",iData);
		return;
	
	}

}

// Element Getter
LayoutParser.prototype.getElement = function(iTag) {
	return this.taggedelements[iTag];
}

// Post process to build links between objects
LayoutParser.prototype.postProcess = function() {

	for (var i=0; i<this.elements.length; i++) {
	
		var aElement = this.elements[i];
		
		for (var j=0; j<this.elementTokens.length; j++) {
	
			this.postProcessFunction = this.elementPost[this.elementTokens[j]];
			
			if (valid(this.postProcessFunction)) {
			
				this.postProcessFunction(aElement);
			
			} 
		
		}
		
	
	}

}


LayoutParser.prototype.postTextInputElement = function(iElement) {
	
	if (valid(iElement.content.follow)) { //optionnal
	
		var aElementToFollow = this.getElement(iElement.content.follow);
		if (valid(aElementToFollow)) {
		
			iElement.element.setFollowing(aElementToFollow.element);
		
		} else {
			logError("LayoutParser : Error on post process, tagged element \""+iElement.content.follow+"\" not defined.",iElement);
			return;
		}
	
	}

}


