/*
 * jQuery history plugin
 * 
 * The MIT License
 * 
 * Copyright (c) 2006-2009 Taku Sano (Mikage Sawatari)
 * Copyright (c) 2010 Takayuki Miwa
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

(function($) {
    var locationWrapper = {
        put: function(hash, win) {
            (win || window).location.hash = this.encoder(hash);
        },
        get: function(win) {
            var hash = ((win || window).location.hash).replace(/^#/, '');
            try {
                return $.browser.mozilla ? hash : decodeURIComponent(hash);
            }
            catch (error) {
                return hash;
            }
        },
        encoder: encodeURIComponent
    };

    var iframeWrapper = {
        id: "__jQuery_history",
        init: function() {
            var html = '<iframe id="'+ this.id +'" style="display:none" src="javascript:false;" />';
            $("body").prepend(html);
            return this;
        },
        _document: function() {
            return $("#"+ this.id)[0].contentWindow.document;
        },
        put: function(hash) {
            var doc = this._document();
            doc.open();
            doc.close();
            locationWrapper.put(hash, doc);
        },
        get: function() {
            return locationWrapper.get(this._document());
        }
    };

    function initObjects(options) {
        options = $.extend({
                unescape: false
            }, options || {});

        locationWrapper.encoder = encoder(options.unescape);

        function encoder(unescape_) {
            if(unescape_ === true) {
                return function(hash){ return hash; };
            }
            if(typeof unescape_ == "string" &&
               (unescape_ = partialDecoder(unescape_.split("")))
               || typeof unescape_ == "function") {
                return function(hash) { return unescape_(encodeURIComponent(hash)); };
            }
            return encodeURIComponent;
        }

        function partialDecoder(chars) {
            var re = new RegExp($.map(chars, encodeURIComponent).join("|"), "ig");
            return function(enc) { return enc.replace(re, decodeURIComponent); };
        }
    }

    var implementations = {};

    implementations.base = {
        callback: undefined,
        type: undefined,

        check: function() {},
        load:  function(hash) {},
        init:  function(callback, options) {
            initObjects(options);
            self.callback = callback;
            self._options = options;
            self._init();
        },

        _init: function() {},
        _options: {}
    };

    implementations.timer = {
        _appState: undefined,
        _init: function() {
            var current_hash = locationWrapper.get();
            self._appState = current_hash;
            self.callback(current_hash);
            setInterval(self.check, 100);
        },
        check: function() {
            var current_hash = locationWrapper.get();
            if(current_hash != self._appState) {
                self._appState = current_hash;
                self.callback(current_hash);
            }
        },
        load: function(hash) {
            if(hash != self._appState) {
                locationWrapper.put(hash);
                self._appState = hash;
                self.callback(hash);
            }
        }
    };

    implementations.iframeTimer = {
        _appState: undefined,
        _init: function() {
            var current_hash = locationWrapper.get();
            self._appState = current_hash;
            iframeWrapper.init().put(current_hash);
            self.callback(current_hash);
            setInterval(self.check, 100);
        },
        check: function() {
            var iframe_hash = iframeWrapper.get(),
                location_hash = locationWrapper.get();

            if (location_hash != iframe_hash) {
                if (location_hash == self._appState) {    // user used Back or Forward button
                    self._appState = iframe_hash;
                    locationWrapper.put(iframe_hash);
                    self.callback(iframe_hash); 
                } else {                              // user loaded new bookmark
                    self._appState = location_hash;  
                    iframeWrapper.put(location_hash);
                    self.callback(location_hash);
                }
            }
        },
        load: function(hash) {
            if(hash != self._appState) {
                locationWrapper.put(hash);
                iframeWrapper.put(hash);
                self._appState = hash;
                self.callback(hash);
            }
        }
    };

    implementations.hashchangeEvent = {
        _init: function() {
            self.callback(locationWrapper.get());
            $(window).bind('hashchange', self.check);
        },
        check: function() {
            self.callback(locationWrapper.get());
        },
        load: function(hash) {
            locationWrapper.put(hash);
        }
    };

    var self = $.extend({}, implementations.base);

    if($.browser.msie && ($.browser.version < 8 || document.documentMode < 8)) {
        self.type = 'iframeTimer';
    } else if("onhashchange" in window) {
        self.type = 'hashchangeEvent';
    } else {
        self.type = 'timer';
    }

    $.extend(self, implementations[self.type]);
    $.history = self;
})(jQuery);
// moment.js
// version : 2.0.0
// author : Tim Wood
// license : MIT
// momentjs.com
(function(e){function O(e,t){return function(n){return j(e.call(this,n),t)}}function M(e){return function(t){return this.lang().ordinal(e.call(this,t))}}function _(){}function D(e){H(this,e)}function P(e){var t=this._data={},n=e.years||e.year||e.y||0,r=e.months||e.month||e.M||0,i=e.weeks||e.week||e.w||0,s=e.days||e.day||e.d||0,o=e.hours||e.hour||e.h||0,u=e.minutes||e.minute||e.m||0,a=e.seconds||e.second||e.s||0,f=e.milliseconds||e.millisecond||e.ms||0;this._milliseconds=f+a*1e3+u*6e4+o*36e5,this._days=s+i*7,this._months=r+n*12,t.milliseconds=f%1e3,a+=B(f/1e3),t.seconds=a%60,u+=B(a/60),t.minutes=u%60,o+=B(u/60),t.hours=o%24,s+=B(o/24),s+=i*7,t.days=s%30,r+=B(s/30),t.months=r%12,n+=B(r/12),t.years=n}function H(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n]);return e}function B(e){return e<0?Math.ceil(e):Math.floor(e)}function j(e,t){var n=e+"";while(n.length<t)n="0"+n;return n}function F(e,t,n){var r=t._milliseconds,i=t._days,s=t._months,o;r&&e._d.setTime(+e+r*n),i&&e.date(e.date()+i*n),s&&(o=e.date(),e.date(1).month(e.month()+s*n).date(Math.min(o,e.daysInMonth())))}function I(e){return Object.prototype.toString.call(e)==="[object Array]"}function q(e,t){var n=Math.min(e.length,t.length),r=Math.abs(e.length-t.length),i=0,s;for(s=0;s<n;s++)~~e[s]!==~~t[s]&&i++;return i+r}function R(e,t){return t.abbr=e,s[e]||(s[e]=new _),s[e].set(t),s[e]}function U(e){return e?(!s[e]&&o&&require("./lang/"+e),s[e]):t.fn._lang}function z(e){return e.match(/\[.*\]/)?e.replace(/^\[|\]$/g,""):e.replace(/\\/g,"")}function W(e){var t=e.match(a),n,r;for(n=0,r=t.length;n<r;n++)A[t[n]]?t[n]=A[t[n]]:t[n]=z(t[n]);return function(i){var s="";for(n=0;n<r;n++)s+=typeof t[n].call=="function"?t[n].call(i,e):t[n];return s}}function X(e,t){function r(t){return e.lang().longDateFormat(t)||t}var n=5;while(n--&&f.test(t))t=t.replace(f,r);return C[t]||(C[t]=W(t)),C[t](e)}function V(e){switch(e){case"DDDD":return p;case"YYYY":return d;case"YYYYY":return v;case"S":case"SS":case"SSS":case"DDD":return h;case"MMM":case"MMMM":case"dd":case"ddd":case"dddd":case"a":case"A":return m;case"X":return b;case"Z":case"ZZ":return g;case"T":return y;case"MM":case"DD":case"YY":case"HH":case"hh":case"mm":case"ss":case"M":case"D":case"d":case"H":case"h":case"m":case"s":return c;default:return new RegExp(e.replace("\\",""))}}function $(e,t,n){var r,i,s=n._a;switch(e){case"M":case"MM":s[1]=t==null?0:~~t-1;break;case"MMM":case"MMMM":r=U(n._l).monthsParse(t),r!=null?s[1]=r:n._isValid=!1;break;case"D":case"DD":case"DDD":case"DDDD":t!=null&&(s[2]=~~t);break;case"YY":s[0]=~~t+(~~t>68?1900:2e3);break;case"YYYY":case"YYYYY":s[0]=~~t;break;case"a":case"A":n._isPm=(t+"").toLowerCase()==="pm";break;case"H":case"HH":case"h":case"hh":s[3]=~~t;break;case"m":case"mm":s[4]=~~t;break;case"s":case"ss":s[5]=~~t;break;case"S":case"SS":case"SSS":s[6]=~~(("0."+t)*1e3);break;case"X":n._d=new Date(parseFloat(t)*1e3);break;case"Z":case"ZZ":n._useUTC=!0,r=(t+"").match(x),r&&r[1]&&(n._tzh=~~r[1]),r&&r[2]&&(n._tzm=~~r[2]),r&&r[0]==="+"&&(n._tzh=-n._tzh,n._tzm=-n._tzm)}t==null&&(n._isValid=!1)}function J(e){var t,n,r=[];if(e._d)return;for(t=0;t<7;t++)e._a[t]=r[t]=e._a[t]==null?t===2?1:0:e._a[t];r[3]+=e._tzh||0,r[4]+=e._tzm||0,n=new Date(0),e._useUTC?(n.setUTCFullYear(r[0],r[1],r[2]),n.setUTCHours(r[3],r[4],r[5],r[6])):(n.setFullYear(r[0],r[1],r[2]),n.setHours(r[3],r[4],r[5],r[6])),e._d=n}function K(e){var t=e._f.match(a),n=e._i,r,i;e._a=[];for(r=0;r<t.length;r++)i=(V(t[r]).exec(n)||[])[0],i&&(n=n.slice(n.indexOf(i)+i.length)),A[t[r]]&&$(t[r],i,e);e._isPm&&e._a[3]<12&&(e._a[3]+=12),e._isPm===!1&&e._a[3]===12&&(e._a[3]=0),J(e)}function Q(e){var t,n,r,i=99,s,o,u;while(e._f.length){t=H({},e),t._f=e._f.pop(),K(t),n=new D(t);if(n.isValid()){r=n;break}u=q(t._a,n.toArray()),u<i&&(i=u,r=n)}H(e,r)}function G(e){var t,n=e._i;if(w.exec(n)){e._f="YYYY-MM-DDT";for(t=0;t<4;t++)if(S[t][1].exec(n)){e._f+=S[t][0];break}g.exec(n)&&(e._f+=" Z"),K(e)}else e._d=new Date(n)}function Y(t){var n=t._i,r=u.exec(n);n===e?t._d=new Date:r?t._d=new Date(+r[1]):typeof n=="string"?G(t):I(n)?(t._a=n.slice(0),J(t)):t._d=n instanceof Date?new Date(+n):new Date(n)}function Z(e,t,n,r,i){return i.relativeTime(t||1,!!n,e,r)}function et(e,t,n){var i=r(Math.abs(e)/1e3),s=r(i/60),o=r(s/60),u=r(o/24),a=r(u/365),f=i<45&&["s",i]||s===1&&["m"]||s<45&&["mm",s]||o===1&&["h"]||o<22&&["hh",o]||u===1&&["d"]||u<=25&&["dd",u]||u<=45&&["M"]||u<345&&["MM",r(u/30)]||a===1&&["y"]||["yy",a];return f[2]=t,f[3]=e>0,f[4]=n,Z.apply({},f)}function tt(e,n,r){var i=r-n,s=r-e.day();return s>i&&(s-=7),s<i-7&&(s+=7),Math.ceil(t(e).add("d",s).dayOfYear()/7)}function nt(e){var n=e._i,r=e._f;return n===null||n===""?null:(typeof n=="string"&&(e._i=n=U().preparse(n)),t.isMoment(n)?(e=H({},n),e._d=new Date(+n._d)):r?I(r)?Q(e):K(e):Y(e),new D(e))}function rt(e,n){t.fn[e]=t.fn[e+"s"]=function(e){var t=this._isUTC?"UTC":"";return e!=null?(this._d["set"+t+n](e),this):this._d["get"+t+n]()}}function it(e){t.duration.fn[e]=function(){return this._data[e]}}function st(e,n){t.duration.fn["as"+e]=function(){return+this/n}}var t,n="2.0.0",r=Math.round,i,s={},o=typeof module!="undefined"&&module.exports,u=/^\/?Date\((\-?\d+)/i,a=/(\[[^\[]*\])|(\\)?(Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|YYYYY|YYYY|YY|a|A|hh?|HH?|mm?|ss?|SS?S?|X|zz?|ZZ?|.)/g,f=/(\[[^\[]*\])|(\\)?(LT|LL?L?L?|l{1,4})/g,l=/([0-9a-zA-Z\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+)/gi,c=/\d\d?/,h=/\d{1,3}/,p=/\d{3}/,d=/\d{1,4}/,v=/[+\-]?\d{1,6}/,m=/[0-9]*[a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+|[\u0600-\u06FF]+\s*?[\u0600-\u06FF]+/i,g=/Z|[\+\-]\d\d:?\d\d/i,y=/T/i,b=/[\+\-]?\d+(\.\d{1,3})?/,w=/^\s*\d{4}-\d\d-\d\d((T| )(\d\d(:\d\d(:\d\d(\.\d\d?\d?)?)?)?)?([\+\-]\d\d:?\d\d)?)?/,E="YYYY-MM-DDTHH:mm:ssZ",S=[["HH:mm:ss.S",/(T| )\d\d:\d\d:\d\d\.\d{1,3}/],["HH:mm:ss",/(T| )\d\d:\d\d:\d\d/],["HH:mm",/(T| )\d\d:\d\d/],["HH",/(T| )\d\d/]],x=/([\+\-]|\d\d)/gi,T="Month|Date|Hours|Minutes|Seconds|Milliseconds".split("|"),N={Milliseconds:1,Seconds:1e3,Minutes:6e4,Hours:36e5,Days:864e5,Months:2592e6,Years:31536e6},C={},k="DDD w W M D d".split(" "),L="M D H h m s w W".split(" "),A={M:function(){return this.month()+1},MMM:function(e){return this.lang().monthsShort(this,e)},MMMM:function(e){return this.lang().months(this,e)},D:function(){return this.date()},DDD:function(){return this.dayOfYear()},d:function(){return this.day()},dd:function(e){return this.lang().weekdaysMin(this,e)},ddd:function(e){return this.lang().weekdaysShort(this,e)},dddd:function(e){return this.lang().weekdays(this,e)},w:function(){return this.week()},W:function(){return this.isoWeek()},YY:function(){return j(this.year()%100,2)},YYYY:function(){return j(this.year(),4)},YYYYY:function(){return j(this.year(),5)},a:function(){return this.lang().meridiem(this.hours(),this.minutes(),!0)},A:function(){return this.lang().meridiem(this.hours(),this.minutes(),!1)},H:function(){return this.hours()},h:function(){return this.hours()%12||12},m:function(){return this.minutes()},s:function(){return this.seconds()},S:function(){return~~(this.milliseconds()/100)},SS:function(){return j(~~(this.milliseconds()/10),2)},SSS:function(){return j(this.milliseconds(),3)},Z:function(){var e=-this.zone(),t="+";return e<0&&(e=-e,t="-"),t+j(~~(e/60),2)+":"+j(~~e%60,2)},ZZ:function(){var e=-this.zone(),t="+";return e<0&&(e=-e,t="-"),t+j(~~(10*e/6),4)},X:function(){return this.unix()}};while(k.length)i=k.pop(),A[i+"o"]=M(A[i]);while(L.length)i=L.pop(),A[i+i]=O(A[i],2);A.DDDD=O(A.DDD,3),_.prototype={set:function(e){var t,n;for(n in e)t=e[n],typeof t=="function"?this[n]=t:this["_"+n]=t},_months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),months:function(e){return this._months[e.month()]},_monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),monthsShort:function(e){return this._monthsShort[e.month()]},monthsParse:function(e){var n,r,i,s;this._monthsParse||(this._monthsParse=[]);for(n=0;n<12;n++){this._monthsParse[n]||(r=t([2e3,n]),i="^"+this.months(r,"")+"|^"+this.monthsShort(r,""),this._monthsParse[n]=new RegExp(i.replace(".",""),"i"));if(this._monthsParse[n].test(e))return n}},_weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdays:function(e){return this._weekdays[e.day()]},_weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysShort:function(e){return this._weekdaysShort[e.day()]},_weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),weekdaysMin:function(e){return this._weekdaysMin[e.day()]},_longDateFormat:{LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D YYYY",LLL:"MMMM D YYYY LT",LLLL:"dddd, MMMM D YYYY LT"},longDateFormat:function(e){var t=this._longDateFormat[e];return!t&&this._longDateFormat[e.toUpperCase()]&&(t=this._longDateFormat[e.toUpperCase()].replace(/MMMM|MM|DD|dddd/g,function(e){return e.slice(1)}),this._longDateFormat[e]=t),t},meridiem:function(e,t,n){return e>11?n?"pm":"PM":n?"am":"AM"},_calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[last] dddd [at] LT",sameElse:"L"},calendar:function(e,t){var n=this._calendar[e];return typeof n=="function"?n.apply(t):n},_relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},relativeTime:function(e,t,n,r){var i=this._relativeTime[n];return typeof i=="function"?i(e,t,n,r):i.replace(/%d/i,e)},pastFuture:function(e,t){var n=this._relativeTime[e>0?"future":"past"];return typeof n=="function"?n(t):n.replace(/%s/i,t)},ordinal:function(e){return this._ordinal.replace("%d",e)},_ordinal:"%d",preparse:function(e){return e},postformat:function(e){return e},week:function(e){return tt(e,this._week.dow,this._week.doy)},_week:{dow:0,doy:6}},t=function(e,t,n){return nt({_i:e,_f:t,_l:n,_isUTC:!1})},t.utc=function(e,t,n){return nt({_useUTC:!0,_isUTC:!0,_l:n,_i:e,_f:t})},t.unix=function(e){return t(e*1e3)},t.duration=function(e,n){var r=t.isDuration(e),i=typeof e=="number",s=r?e._data:i?{}:e,o;return i&&(n?s[n]=e:s.milliseconds=e),o=new P(s),r&&e.hasOwnProperty("_lang")&&(o._lang=e._lang),o},t.version=n,t.defaultFormat=E,t.lang=function(e,n){var r;if(!e)return t.fn._lang._abbr;n?R(e,n):s[e]||U(e),t.duration.fn._lang=t.fn._lang=U(e)},t.langData=function(e){return e&&e._lang&&e._lang._abbr&&(e=e._lang._abbr),U(e)},t.isMoment=function(e){return e instanceof D},t.isDuration=function(e){return e instanceof P},t.fn=D.prototype={clone:function(){return t(this)},valueOf:function(){return+this._d},unix:function(){return Math.floor(+this._d/1e3)},toString:function(){return this.format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")},toDate:function(){return this._d},toJSON:function(){return t.utc(this).format("YYYY-MM-DD[T]HH:mm:ss.SSS[Z]")},toArray:function(){var e=this;return[e.year(),e.month(),e.date(),e.hours(),e.minutes(),e.seconds(),e.milliseconds()]},isValid:function(){return this._isValid==null&&(this._a?this._isValid=!q(this._a,(this._isUTC?t.utc(this._a):t(this._a)).toArray()):this._isValid=!isNaN(this._d.getTime())),!!this._isValid},utc:function(){return this._isUTC=!0,this},local:function(){return this._isUTC=!1,this},format:function(e){var n=X(this,e||t.defaultFormat);return this.lang().postformat(n)},add:function(e,n){var r;return typeof e=="string"?r=t.duration(+n,e):r=t.duration(e,n),F(this,r,1),this},subtract:function(e,n){var r;return typeof e=="string"?r=t.duration(+n,e):r=t.duration(e,n),F(this,r,-1),this},diff:function(e,n,r){var i=this._isUTC?t(e).utc():t(e).local(),s=(this.zone()-i.zone())*6e4,o,u;return n&&(n=n.replace(/s$/,"")),n==="year"||n==="month"?(o=(this.daysInMonth()+i.daysInMonth())*432e5,u=(this.year()-i.year())*12+(this.month()-i.month()),u+=(this-t(this).startOf("month")-(i-t(i).startOf("month")))/o,n==="year"&&(u/=12)):(o=this-i-s,u=n==="second"?o/1e3:n==="minute"?o/6e4:n==="hour"?o/36e5:n==="day"?o/864e5:n==="week"?o/6048e5:o),r?u:B(u)},from:function(e,n){return t.duration(this.diff(e)).lang(this.lang()._abbr).humanize(!n)},fromNow:function(e){return this.from(t(),e)},calendar:function(){var e=this.diff(t().startOf("day"),"days",!0),n=e<-6?"sameElse":e<-1?"lastWeek":e<0?"lastDay":e<1?"sameDay":e<2?"nextDay":e<7?"nextWeek":"sameElse";return this.format(this.lang().calendar(n,this))},isLeapYear:function(){var e=this.year();return e%4===0&&e%100!==0||e%400===0},isDST:function(){return this.zone()<t([this.year()]).zone()||this.zone()<t([this.year(),5]).zone()},day:function(e){var t=this._isUTC?this._d.getUTCDay():this._d.getDay();return e==null?t:this.add({d:e-t})},startOf:function(e){e=e.replace(/s$/,"");switch(e){case"year":this.month(0);case"month":this.date(1);case"week":case"day":this.hours(0);case"hour":this.minutes(0);case"minute":this.seconds(0);case"second":this.milliseconds(0)}return e==="week"&&this.day(0),this},endOf:function(e){return this.startOf(e).add(e.replace(/s?$/,"s"),1).subtract("ms",1)},isAfter:function(e,n){return n=typeof n!="undefined"?n:"millisecond",+this.clone().startOf(n)>+t(e).startOf(n)},isBefore:function(e,n){return n=typeof n!="undefined"?n:"millisecond",+this.clone().startOf(n)<+t(e).startOf(n)},isSame:function(e,n){return n=typeof n!="undefined"?n:"millisecond",+this.clone().startOf(n)===+t(e).startOf(n)},zone:function(){return this._isUTC?0:this._d.getTimezoneOffset()},daysInMonth:function(){return t.utc([this.year(),this.month()+1,0]).date()},dayOfYear:function(e){var n=r((t(this).startOf("day")-t(this).startOf("year"))/864e5)+1;return e==null?n:this.add("d",e-n)},isoWeek:function(e){var t=tt(this,1,4);return e==null?t:this.add("d",(e-t)*7)},week:function(e){var t=this.lang().week(this);return e==null?t:this.add("d",(e-t)*7)},lang:function(t){return t===e?this._lang:(this._lang=U(t),this)}};for(i=0;i<T.length;i++)rt(T[i].toLowerCase().replace(/s$/,""),T[i]);rt("year","FullYear"),t.fn.days=t.fn.day,t.fn.weeks=t.fn.week,t.fn.isoWeeks=t.fn.isoWeek,t.duration.fn=P.prototype={weeks:function(){return B(this.days()/7)},valueOf:function(){return this._milliseconds+this._days*864e5+this._months*2592e6},humanize:function(e){var t=+this,n=et(t,!e,this.lang());return e&&(n=this.lang().pastFuture(t,n)),this.lang().postformat(n)},lang:t.fn.lang};for(i in N)N.hasOwnProperty(i)&&(st(i,N[i]),it(i.toLowerCase()));st("Weeks",6048e5),t.lang("en",{ordinal:function(e){var t=e%10,n=~~(e%100/10)===1?"th":t===1?"st":t===2?"nd":t===3?"rd":"th";return e+n}}),o&&(module.exports=t),typeof ender=="undefined"&&(this.moment=t),typeof define=="function"&&define.amd&&define("moment",[],function(){return t})}).call(this);
// Adding inner CSS Definition to prevent including external CSS
var innercss = '.gLB { position: absolute; } .gLC {position: absolute; padding-bottom: 0px; margin: 0px; padding-left: 0px; padding-right: 0px; padding-top: 0px;}',
    head = document.getElementsByTagName('head')[0],
    style = document.createElement('style');

style.type = 'text/css';
if (style.styleSheet){
  style.styleSheet.cssText = innercss;
} else {
  style.appendChild(document.createTextNode(innercss));
}

head.appendChild(style);// For IE 
if (typeof console==="undefined") {
	window.console = {
        log: function () {}
    };
}

// Again IE !!
if(!Array.indexOf){  
	Array.prototype.indexOf = function(obj){  
	    for(var i=0; i<this.length; i++){  
	        if(this[i]==obj){  
	            return i;  
	        }  
	    }  
	    return -1;  
	}  
}  

// TOOL  // TODO really needs to create a tool library for that
function valid(iField){
	return !(typeof iField==="undefined");
}

function logError(iString,iData){
	console.log(iString);
	console.log(iData);
	alert(iString);
}

// this is there to select nearest descendent of selector
$.fn.nearest = function(selector) {
	
	var nearest = $(), node = this, distance = 10000;
	
	node.find(selector).each( function(){
	
		//console.log("Examine:");
		//console.log(this);
	
		var n= $(this),
			d= n.parentsUntil(node).size();
		//console.log("Distance:"+d);
		
		if (d<distance) {
			//console.log("Push");
			distance = d;
			nearest.push(this);
		} else if (d==distance) {
			//console.log("Add");
			nearest.push(this);
		}
		
	});
	
	//console.log("NEAREST");
	//console.log(nearest);
	
	return nearest;

}

// exists
$.fn.exists = function(){return this.length>0;}

// jQuery solution to setup a caret at a position
$.fn.selectRange = function(start, end) {
    return this.each(function() {
        if (this.setSelectionRange) {
            this.focus();
            this.setSelectionRange(start, end);
        } else if (this.createTextRange) {
            var range = this.createTextRange();
            range.collapse(true);
            range.moveEnd('character', end);
            range.moveStart('character', start);
            range.select();
        }
    });
};

// TODO to review both functions against != attributes across browsers
$.fn.disableSelection = function() {
        return this
                 .attr('unselectable', 'on')
                 .css('user-select', 'none')
                 .on('selectstart', false);
};

$.fn.enableSelection = function() {
        return this
                 .attr('unselectable','off')
                 .css('user-select', 'text')
                 .on('selectstart', true);
};

$.cssHooks.backgroundColor = {
    get: function(elem) {
        if (elem.currentStyle)
            var bg = elem.currentStyle["backgroundColor"];
        else if (window.getComputedStyle)
            var bg = document.defaultView.getComputedStyle(elem,
                null).getPropertyValue("background-color");
        if (bg.search("rgb") == -1) {
            return bg;
        } else if (bg.search("rgba") !=-1) { // Correction for chrome
        	
        	bg = bg.match(/^rgba\((\d+),\s*(\d+),\s*(\d+),\s*(\d+)\)$/);
            function hex(x) {
                return ("0" + parseInt(x).toString(16)).slice(-2);
            }
            //console.log("#" + hex(bg[1]) + hex(bg[2]) + hex(bg[3]) );
            if (bg[4]==0) { // Means Alpha = 0 => no color
            	return;
            } else {
            	return "#" + hex(bg[1]) + hex(bg[2]) + hex(bg[3]);
            }
            
        } else {
            
            bg = bg.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);
            function hex(x) {
                return ("0" + parseInt(x).toString(16)).slice(-2);
            }
            return "#" + hex(bg[1]) + hex(bg[2]) + hex(bg[3]);
        
        }
    }
}

// Enable navigation management

var hashFunctionToCall = [];

var HashCall = {};

HashCall.pushHashFunction = function(iFunction) {
	hashFunctionToCall.push(iFunction);
}

$(document).ready(function() {
   
    $.history.init(function(hash){
        if(hash == "") {
            // initialize your app
            //console.log("The app is initialized");
        } else {
            // restore the state from hash
        	//console.log("Go to hash "+hash);
        	for (var i=0;i<hashFunctionToCall.length; i++) {
        		hashFunctionToCall[i].call(this, hash);
        	}
        	
        }
    },
    { unescape: "#=" });
   

});

$.fn.textWidth = function(text){
  var org = $(this);
  var html = $('<div "position:relative;overflow:hidden;"><span style="position:absolute;width:auto;height:auto;left:-9999px;visibility:hidden;white-space:nowrap;">' + (text || org.html()) + '</span></div>');
  if (!text) {
    html.css("font-family", org.css("font-family"));
    html.css("font-size", org.css("font-size"));
  }
  $('body').append(html);
  var width = html.find("span").width();
  html.remove();
  return width;
}

$.fn.textHeight = function(text){
  var org = $(this);
  var html = $('<div "position:relative;overflow:hidden;"><span style="position:absolute;width:auto;height:auto;left:-9999px;visibility:hidden;white-space:nowrap;">' + (text || org.html()) + '</span></div>');
  if (!text) {
    html.css("font-family", org.css("font-family"));
    html.css("font-size", org.css("font-size"));
  }
  $('body').append(html);
  var height = html.find("span").height();
  html.remove();
  return height;
}

// Split location

function getHashSplit(iHash){

	var result = {};
	var elements = ("#"+iHash).split("#");	
	if (elements.length>1) {
		
		for (var i=1;i<elements.length; i++) {
		
			var keyValue = elements[i].split("=");
			result[keyValue[0]] = keyValue[1];
		
		}
		
	}
	return result;

}

function getLocationSplit(){

	var result = {};
	var elements = window.location.href.toString().split("#");	
	if (elements.length>1) {
		
		for (var i=1;i<elements.length; i++) {
		
			var keyValue = elements[i].split("=");
			result[keyValue[0]] = keyValue[1];
		
		}
		
	}
	return result;

}



function buildLocationHash(iElements){
	var hash = "";
	for (var i in iElements) {
		hash = hash+ "#" +i+ "=" +iElements[i] ;
	}
	return hash;
}

String.prototype.hashCode = function(){
    var hash = 0, i, achar;
    if (this.length == 0) return hash;
    for (i = 0; i < this.length; i++) {
        achar = this.charCodeAt(i);
        hash = ((hash<<5)-hash)+achar;
        hash = hash & hash; // Convert to 32bit integer
    }
    return hash.toString(16);
};


// Size of an object
Object.size = function(obj) {
    var size = 0, key;
    for (key in obj) {
        if (obj.hasOwnProperty(key)) size++;
    }
    return size;
};



function loadcallback() {
	console.log( "load js file ok" );
}

function importJS(filename){
		
	$.ajaxSetup({async:false}); // This is removed to be sure that the file and it's definition 
								// will be applied to all the script
	$.getScript(filename, loadcallback )
	.done(function(script, textStatus) {
		console.log( textStatus );
	})
	.fail(function(jqxhr, settings, exception) {
		console.log( "Error when loading import "  );
		console.log(exception);
		
		var vDebug = ""; 
		for (var prop in exception) 
		{  
		   vDebug += "property: "+ prop+ " value: ["+ exception[prop]+ "]\n"; 
		} 
		vDebug += "toString(): " + " value: [" + exception.toString() + "]"; 
		console.log(vDebug)
		
	});
	$.ajaxSetup({async:true});
		
}
		
function importCSS(filename){

	if (document.createStyleSheet){
		document.createStyleSheet("'"+filename+"'");
	} else {
		$("head").append($("<link rel='stylesheet' href='"+filename+"' type='text/css' media='screen' />"));
	}

}
// Static class
var International = function(){
} 

var internationalData = {};

International.load = function(iFile){

	//console.log("Load international ...");

	internationalData = {};
	
	$.ajaxSetup( { "async": false } );
	$.getJSON(iFile, function(data) {
	  
	  $.each(data, function(key, val) {
	    //console.log("key:"+key + " value:" + val + "");
		
		var newValue = {};
		$.each(val, function(keyText, valueText) {
			//console.log("keyText:"+keyText + " valueText:" + valueText + "");
			newValue[keyText]=valueText;
		});
		
		internationalData[key] = newValue;
		
	  });
	
	});
	$.ajaxSetup( { "async": true } );

}

International.listDefinedLanguages = function(){
	
	var aList = [];
	$.each( internationalData, function( key, value ) {
		aList.push(key);
	});
	return aList;
	
}

International.translate = function( iLanguage, iField, iDefault ){

	var value = iDefault;
	if (!(typeof internationalData[iLanguage] === "undefined" )) {
	
		if (!(typeof internationalData[iLanguage][iField] === "undefined" )) {
			 value = internationalData[iLanguage][iField];
		}
	
	}

	return value;
	
}
var toBeResized = [];

function handleResize() {

	//console.log("Resize");
	
	for (var i=0; i<toBeResized.length; i++) {
		//console.log(toBeResized[i]);
		if (!(typeof toBeResized[i] ==="undefined")) {
			toBeResized[i](); 
		}
	}
	
};

function hasToBeResized(iFunction){
	toBeResized.push(iFunction);	
}

var resizeTimer;
var winWidth = $(window).width(),
    winHeight = $(window).height();

$(window).resize(function() {
	//console.log("Resize requested");
	
	// This is added to prevent infinite loop in IE
	//New height and width
    var winNewWidth = $(window).width(),
    winNewHeight = $(window).height();

    // compare the new height and width with old one
    if(winWidth!=winNewWidth || winHeight!=winNewHeight)
    {
        clearTimeout(resizeTimer);
		resizeTimer = setTimeout(handleResize, 30);
    }
    //Update the width and height
    winWidth = winNewWidth;
    winHeight = winNewHeight;


});

// ######################## DEFINE ELEMENT     #########################
var elementID = 0;

var ELEMENTCLASSLIST = ".gridlayout,.containerElement,.scrollContainerElement,.textelement,.textinputelement,.imgelement,.linearlistlayout,.mapperelement,.verticalbarelement,.buttonelement,.sliderelement";

var Element = function(){
	this.element=null;
} 

Element.prototype.implement = function () {
	//console.log("Implement function element not implemented");
}

Element.prototype.afterInsert = function() {
	//console.log("afterInsert function element not implemented");
}

Element.prototype.setProperties = function (iProperties){
	this.prop=iProperties;
	return this;
}

Element.prototype.getInnerElements = function () {
	return [];
}

Element.markElement = function (iElement, iPropertyName, iProperty) {

	//console.log("Mark element :");
	//console.log(iElement);
	//console.log(iElement.element);
	iElement[iPropertyName] = iProperty;
	if (valid(iElement.element)) {
		$(iElement.element).data(iPropertyName,iProperty);
	}
	var elements = iElement.getInnerElements();
	//console.log("Number of inner elements:"+elements.length);
	for (var i=0; i<elements.length; i++) {
		Element.markElement(elements[i], iPropertyName, iProperty);
	}
	
}

Element.prototype.shiftZIndex = function (iIndex) {

	var aElement = $(this.element);
	var currentZIndex = aElement.css("z-index");
	aElement.css( 'z-index' , currentZIndex + iIndex );
	
	$.each( aElement.find(ELEMENTCLASSLIST),
			function(key,value) {
				var currentZIndex = $(value).css("z-index");
				$(value).css( 'z-index' , currentZIndex + iIndex );
			}
	);
	
	return this;

}

Element.prototype.hide = function () {

	var aElement = $(this.element);
	aElement.css("visibility","hidden");
	
	$.each( aElement.find(ELEMENTCLASSLIST),
			function(key,value) {
				$(value).css("visibility","hidden");
			}
	);
	
	return this;

}

Element.prototype.show = function () {

	var aElement = $(this.element);
	aElement.css("visibility","visible");
	
	$.each( aElement.find(ELEMENTCLASSLIST),
			function(key,value) {
				$(value).css("visibility","visible");
			}
	);

	return this;
	
}



Element.prototype.checkWrapping = function (iInnerElement){

	var propDefined = (!(typeof this.prop==="undefined")) && (this.prop!=null);
	// This will validate if the cell element is a container
	// If we are in wrap mode for the layout/container, we have to raise error/or setup container to wrap mode
	// Best is to set error to indicate an issue with the layout definition
	if (propDefined?!(typeof this.prop.wrap==="undefined"):false){

		if (this.prop.wrap==true) {
		
			var inError = true;
		
			if (iInnerElement instanceof ContainerElement) {
			
				if (!(typeof iInnerElement.prop==='undefined')) {
					
					if (!(typeof iInnerElement.prop.wrap==='undefined')){
					  if (iInnerElement.prop.wrap==true) {
						inError = false;
					  }
					} 
				
				} 
			
			} else if (iInnerElement instanceof ImageElement) {
			
				if (!(typeof iInnerElement.prop==='undefined')) {
					
					if (!(typeof iInnerElement.prop.wrap==='undefined')){
					  if (iInnerElement.prop.wrap==true) {
						inError = false;
					  }
					} 
				
				}

			} else { // The element is not a container nor an image, so self define its size
				inError = false;
			}
			
			if (inError) {
			
				console.log("Error: unable to wrap the container/image, please setup wrap mode for the container/image.");
				return new TextElement("#ERROR#");
			
			}
	
		}
		
	}
	
	return iInnerElement;

}

Element.resizeInChain = function (iObject, iElement, iResizeChain, iForceResize) {
	
	var id = $(iElement).attr('id')+"|";
	
	if (!valid(iResizeChain)) {
		iResizeChain = "|" ;
	}
	
	if (iResizeChain.length==1) {
	
		iResizeChain = iResizeChain + id ;
		iObject.resize(iElement, iResizeChain, iForceResize);
		
	} else {
	
		if ((iResizeChain.indexOf("|"+id)==-1)) {
		
			iResizeChain = iResizeChain + id ;
			iObject.resize(iElement, iResizeChain, iForceResize);
		
		} else {
		
			iResizeChain = iResizeChain + id ; 
			console.log("Detected loop:"+iResizeChain);
		
		}
	
	}
	
}

Element.prototype.insert = function (iElement) {
	var aElement = $(iElement);
	$(aElement).empty();
	var html_code = this.implement();
	$(aElement).append(html_code);
	this.afterInsert();
	handleLayoutResize();
}

Element.resizeSpecialElements = function(iElement,iChain,iForceResize) {

	//console.log("Resize special element ...");
	// Here we ask to resize ScrollContainerElement if needed
	var aElement = $(iElement).parent().closest(ELEMENTCLASSLIST);
	if (valid(aElement)) {
	
		//console.log("Element");
		//console.log(aElement);
		if (aElement.hasClass( 'scrollContainerElement' )) {
			Element.resizeInChain(ScrollContainerElement,aElement,iChain,iForceResize);
		}
	
	}

}
// ######################## DEFINE ELEMENT IMAGE ########################
var ImageElement = function(iSrc){
	Element.call(this);
	this.src = iSrc;
} 
ImageElement.prototype = new Element();
ImageElement.prototype.constructor = ImageElement;
	
ImageElement.prototype.setPosition = function (iPosition){
	this.position = iPosition;
	return this;
}

ImageElement.prototype.implement = function () {

	// at that step we have the image size
	/*console.log("Waited width :"+waitImage.width);
	//console.log("Waited height :"+waitImage.height);*/
	this.width = 0;
	this.height = 0;
	
	var html_code = "<div id='elem"+elementID+"' class='imgelement' style='";
	if (!(typeof this.position==="undefined")) {
		if (this.position=='center') {
			html_code = html_code + "text-align:center;"; // have to define the size
		} else if (this.position=='left') {
			html_code = html_code + "text-align:left;"; 
		} else if (this.position=='right') {
			html_code = html_code + "text-align:right;";
		}

	}
	var wrap = (!(typeof prop==="undefined")?prop.wrap:false);
	if (wrap) {
		html_code = html_code + "width:"+this.width+"px;"+"height:"+this.height+";";
	}
	html_code = html_code + "'><img  src='"+this.src+"' style='position:absolute' ></div>";

	this.id = elementID;
	this.element = "#elem"+elementID;
	elementID++;
	return html_code;
}

ImageElement.prototype.onClick = function (iOnClickFunction){
	this.onClickFunction = iOnClickFunction;
	return this;
}

ImageElement.afterInsertOnLoad = function(iElement,iWidth,iHeight) {

	$(iElement).data("imgwidth",iWidth);
	$(iElement).data("imgheight",iHeight);
	$(iElement).data("loaded",true);

	//console.log("ImageElement:"+$(iElement).attr('id'));
	//console.log("Width:"+iWidth);
	//console.log("Height:"+iHeight);
	
	ImageElement.resize($(iElement),"",true);
	
	$(iElement).show();	
	
}

ImageElement.onLoadFunction = function(iElement,iImage){
	return function(){
		//console.log("ImageElement:"+$(iElement).attr('id'));
		//console.log('Image loaded width:'+iImage.width);
		//console.log('Image loaded height:'+iImage.height);
		ImageElement.afterInsertOnLoad(iElement,iImage.width,iImage.height);
	}
}

ImageElement.prototype.afterInsert = function() {

	var aElement = this.element;
	
	if (valid(this.attached)) {
		$(aElement).data("attached",this.attached);
	}
	
	$(aElement).hide();
	
    var waitImage = new Image();
    waitImage.onload = ImageElement.onLoadFunction(aElement, waitImage);
    waitImage.src = this.src;
	
	$(this.element).data("prop",this.prop);
	if (valid(this.onClickFunction)) {
		console.log("On click function installed");
		$(this.element).click(this.onClickFunction);
		$(this.element).css('cursor', 'pointer');
	}
	
}

ImageElement.resize = function (iElement,iChain,iForceResize){
	
	
	var aElement = $(iElement);
	var imgWidth = aElement.data("imgwidth");
	var imgHeight = aElement.data("imgheight");
	var prop = aElement.data("prop");
	
	var ratio = imgWidth/imgHeight;
	
	var wrap = (!(typeof prop==="undefined")?prop.wrap:false);
	if (!wrap) {
		var elem = aElement.data("parentWrap");
		wrap = (typeof elem==="undefined"?false:elem);
	}
	
	if (!wrap) {
	
		var aParent = aElement.parent();
		var estWidth = aParent.width();
		var estHeight = Math.floor(estWidth/ratio);
		
		//console.log("Before EstWidth:"+estWidth);
		//console.log("Before EstHeight:"+estHeight);
		
		aElement.width(estWidth);
		aElement.height(aParent.height());
		
		if (estHeight>aParent.height()) {
		
			estHeight = aParent.height();
			estWidth = Math.floor(estHeight*ratio);
			
		}
		
		var aTop = (aParent.height()-estHeight)/2;
		var aLeft = (aParent.width()-estWidth)/2;
		
		aElement.find('img').css({ "top" : aTop, "left" : aLeft });
		
		//console.log("ratio:"+ratio);
		//console.log("EstWidth:"+estWidth);
		//console.log("EstHeight:"+estHeight);
		
		aElement.find('img').width(estWidth);
		aElement.find('img').height(estHeight);
		
	} else {
	
		aElement.width(imgWidth);
		aElement.height(imgHeight);
		
	}
	
	Element.resizeSpecialElements(iElement,iChain,iForceResize);
	
}// ######################## DEFINE ELEMENT TEXT ########################
var TextElement = function(iText){
	Element.call(this);
	iText = iText.replace(/ /g,"&nbsp;"); 
	this.text = iText;
} 
TextElement.prototype = new Element();
TextElement.prototype.constructor = TextElement;

TextElement.hashcall = function(iHash) {
	
	//console.log("TextElement check for page load ...");
	//console.log(iHash);
	
	var aSplit = getHashSplit(iHash);
	
	var aUrl;
	$.each( $(document).find(".textelement") , function(key,value) {
	
		var hashes = $(value).data("hashes");
		if (valid(hashes)){
		
			if (Object.size(hashes)>0) {
				//console.log("Text element contains hashes:");
				//console.log(value);
				if (valid(hashes[aSplit.HA])) {
					aUrl = hashes[aSplit.HA];
				}
			}
			
		}
		
	});
	
	if (valid(aUrl)) {
	
		//console.log("Load item from url :");
		var aTargetElement = $("#"+aSplit.LTO);
		var aTextElement = aTargetElement.closest(".textelement");
		aTextElement.data('ishtmltext',true);
			
		var linkColor = aTextElement.data('linkcolor');
		var aContext = {};
		aContext.linkColor = linkColor;
		var textValue = TextElement.load(aUrl);
	
		textValue = TextElement.parseHTML(aContext,textValue);
		aContext.id = "#"+aTextElement.attr('id');
	
		aTargetElement.empty();
		aTargetElement.append(textValue);
		
		TextElement.postInsertBuildLinks(aContext);
		
		TextElement.resize(aTextElement);
	
	}
	
}

HashCall.pushHashFunction(TextElement.hashcall);

TextElement.prototype.isUrl = function(){
	this.url = true;
	this.htmltext = true;
}

TextElement.prototype.isHTMLText = function(){
	this.htmltext = true;
}


TextElement.prototype.setFontSize = function (iSize){
	this.size = iSize;
	return this;
}
	
TextElement.prototype.setStyle = function (iStyle){
	this.style = iStyle;
	return this;
}	
	
TextElement.prototype.setPosition = function (iPosition){
	this.position = iPosition;
	return this;
}

TextElement.prototype.setVerticalPosition = function (iPosition){
	this.vposition = iPosition;
	return this;
}

TextElement.prototype.setColor = function (iColor){
	this.color = iColor;
	return this;
}

TextElement.prototype.onClick = function (iOnClickFunction){
	this.onClickFunction = iOnClickFunction;
	return this;
}

TextElement.prototype.link = function (iLink){
	this.linked = iLink;
	return this;
}

TextElement.prototype.setMouseOverColor = function (iColor){
	this.onMouseOverColor = iColor;
	return this;
}

TextElement.prototype.setMouseOverBkColor = function (iColor){
	this.onBkMouseOverColor = iColor;
	return this;
}

TextElement.prototype.setLinkColor = function (iColor){
	this.linkColor = iColor;
	return this;
}

TextElement.prototype.translate = function(iField){
	this.translate = iField;
	return this;
}

TextElement.prototype.implement = function () {
    this.id  = elementID;
	this.element = "#elem"+elementID;
	var html_code = "<div id='elem"+elementID+"' class='textelement ui-widget' style='text-align:justify; position:absolute; top:0px; left:0px;  ";
	elementID++;
	if (valid(this.size)) {
		html_code = html_code + "font-size:"+this.size+";";
	}

	if (!(typeof this.color==="undefined")) {
		html_code = html_code + "color : "+this.color+";";
	}
	
	if (valid(this.style)) {
		html_code = html_code + this.style;
	}
	
	var textValue = this.text;
	if (!(typeof this.translate==="undefined")) {
		textValue = International.translate( Layout.language , this.translate, textValue);
	}
	
	if (this.url) {
	
		//console.log("The text represent an url");
		textValue = TextElement.load(textValue);
	
	}
	
	var aContext={};
	aContext.linkColor = this.linkColor;
	textValue = TextElement.parseHTML(aContext,textValue);
	aContext.id = this.element;
	//console.log("Context:");
	//console.log(aContext);
	this.context = aContext;
	
	if (!(typeof this.linked==="undefined")) {
		html_code = html_code + "'><a href='"+this.linked+"'>"+textValue+"</a></div>";
	} else {
		html_code = html_code + "'>"+textValue+"</div>";
	}
	return html_code;
}

// Here is the text parser that handle small internal functionnalities
// tag {r:[ref]:[url]}[Text]{:} indicate to load data from url inside ref text element
// tag {d:[ref]}                declares a reference, i.e. text will be included there
// tag {f:[function]}		    call a function
// This is a functionnality outside the layout parser to work quickly (Quick and Dirty)

TextElement.refNumber = 0;

TextElement.parseTAG = function(iContext,iHTML,iTagType){

	var aHtml = "";

	if (iTagType=="r") { // define a link with inner text
		
		// extract the tag name
		var aRef = "";
		s = iHTML.indexOf(":");
		if (s==-1) {
			logError("TextElement : no ref defined from url file.");
			return;
		}
		aRef =  iHTML.substring(0,s);
		iHTML = iHTML.substring(s+1);
		
		// extract the url
		var aUrl = "";
		s = iHTML.indexOf("}");
		if (s==-1) {
			logError("TextElement : no url defined .");
			return;
		}
		aUrl = iHTML.substring(0,s);
		iHTML = iHTML.substring(s+1);
		
		iContext.refs.push({
			TY	: 'r',
			ID  : 'ref'+TextElement.refNumber,
			TO  : aRef,
			URL : aUrl 
		});
		
		iContext.hashes[aUrl.hashCode()] = aUrl;
		
		aHtml = "<span id='ref"+TextElement.refNumber+"' ";
		if (valid(iContext.linkColor)) {
			aHtml += "style='color:"+iContext.linkColor+"'>";
		} else {
			aHtml += ">";
		}
		TextElement.refNumber++;
		
	} else if (iTagType=="f") {
	
		// extract the function name
		var aFunction = "";
		s = iHTML.indexOf("}");
		if (s==-1) {
			logError("TextElement : no function defined .");
			return;
		}
		aFunction = iHTML.substring(0,s);
		iHTML = iHTML.substring(s+1);
		
		iContext.refs.push({
			TY	: 'f',
			ID  : 'ref'+TextElement.refNumber,
			FU  : aFunction
		});
	
		aHtml = "<span id='ref"+TextElement.refNumber+"' ";
		if (valid(iContext.linkColor)) {
			aHtml += "style='color:"+iContext.linkColor+"'>";
		} else {
			aHtml += ">";
		}
		TextElement.refNumber++;
	
	} else if (iTagType=="d") { // define a reference
		
		// extract the url
		var aRef = "";
		s = iHTML.indexOf("}");
		if (s==-1) {
			logError("TextElement : no reference defined .");
			return;
		}
		aRef =  iHTML.substring(0,s);
		iHTML = iHTML.substring(s+1);
		
		aHtml = "<span id='"+aRef+"'></span>";
		
	} else if (iTagType=="") { // closing tab i.e  {:}
		
		s = iHTML.indexOf("}");
		if (s==-1) {
			logError("TextElement : no closing bracket defined.");
			return;
		}
		iHTML = iHTML.substring(s+1);
		
		aHtml = "</span>";
		
	} else {
		logError("TextElement : invalid tag '"+iTagType+"'.");
		return;
	}

	return {
		"entry" : iHTML,
		"html"  : aHtml
	};
	
}

TextElement.parseHTML = function(iContext,iHTML){

	var html_code = "";
	
	if (typeof iHTML==="number") {
		iHTML = iHTML.toString();
	}
	
	iContext.refs = [];
	iContext.hashes = {};

	var s = iHTML.indexOf("{");
	while (s!=-1) {
	
		if ((s==0) || (iHTML.charAt(s-1)!="\\")) { // this permit to define '{' => '\{'
		
			// Remove entry character
			html_code += iHTML.substring(0,s);
			iHTML = iHTML.substring(s+1);
		
			// Find Tag type
			var aTagType = "";
			s = iHTML.indexOf(":");
			aTagType =  iHTML.substring(0,s);
			iHTML = iHTML.substring(s+1);
			
			var result = TextElement.parseTAG(iContext,iHTML,aTagType);
			
			if (valid(result)) {
				iHTML = result.entry;
				html_code += result.html;
			}
		
		} else {
		
			// We have to skip the parsing
			html_code += iHTML.substring(0,s-1) + "{" ;
			iHTML = iHTML.substring(s+1);
		
		}
		
		s = iHTML.indexOf("{");
		
	}
	html_code = html_code + iHTML;

	return html_code;

}

TextElement.elementLinkBuilder = function(iUrlHash,iTo) {
	return function() {
		
		// Build the event to call
		//console.log("To id:"+iTo);
		//console.log("Generated hash for url:"+iUrlHash);
		
		// Here call the history
		var aSplit = getLocationSplit();
		aSplit.LTO = iTo;
		aSplit.HA = iUrlHash;
		$.history.load(buildLocationHash(aSplit));
		
	};
}

TextElement.postInsertBuildLinks = function(iContext) {

	if (valid(iContext.refs)) {
	
		//console.log('Add hashes to the element:'+iContext.id);
		$(iContext.id).data("hashes",iContext.hashes);
	
		for (var i=0; i<iContext.refs.length; i++) {
			
			var aElement = $("#"+iContext.refs[i].ID);
			//console.log(iContext.refs[i]);
			//console.log(aElement);
			
			if (valid(aElement)) {
				if (iContext.refs[i].TY=="r") {
					aElement.click(TextElement.elementLinkBuilder(iContext.refs[i].URL.hashCode(),iContext.refs[i].TO));
				} else if (iContext.refs[i].TY=="f") {
					// pass function name
					aElement.click(eval(iContext.refs[i].FU));
				}
				aElement.css('cursor', 'pointer');
			}
			
		}
	
	}

}

TextElement.load = function(iUrl) {
	
	var html_code = "";
	$.ajaxSetup( { "async": false } );
	$.get(iUrl, function(data) {
		//console.log(data);
		html_code = data;
	},"html");
	$.ajaxSetup( { "async": true } );
	
	return html_code;

}

TextElement.changeLanguage = function(iElement){

	var aElement = $(iElement);
	var linked = aElement.data("linked");
	var translate = aElement.data("translate");
	
	var textValue = aElement.data("text");
	//console.log("Element");
	//console.log(aElement);
	//console.log("Text value:"+textValue);
	if (!(typeof translate==="undefined")) {
	
		textValue = International.translate( Layout.language , translate, textValue);
	
		var isUrl = aElement.data("isurl");
		if (isUrl) {
			//console.log("The text represent an url");
			textValue = TextElement.load(textValue);
		}
		
		var aContext={};
		aContext.linkColor = aElement.data("linkcolor");
		textValue = TextElement.parseHTML(aContext,textValue); // TODO Find a way to retrieve text element
		aContext.id = aElement.attr('id');
		
		var html_code = "";
		
		if (!(typeof linked==="undefined")) {
			html_code = html_code + "<a href='"+linked+"'>"+textValue+"</a>";
		} else {
			html_code = html_code + ""+textValue+"";
		}
		
		aElement.empty();
		aElement.append(html_code);
		
		TextElement.postInsertBuildLinks(aContext);
	
	}
	
}

TextElement.onMouseEnterCB = function(aElement){

	//console.log("Mouse Enter");
	var color = $(aElement).data('onMouseOverColor');
	var bkcolor = $(aElement).data('onBkMouseOverColor');
	if (valid(color)) {$(aElement).css("color",color);}
	if (valid(bkcolor)) {$(aElement).css("backgroundColor",bkcolor);}

}

TextElement.onMouseOutCB = function(aElement){

	//console.log("Mouse Out");
	var color = $(aElement).data('color');
	var bkcolor = $(aElement).data('bkcolor');
	if (valid(color)) {$(aElement).css("color",color);}
	if (valid(bkcolor)) {$(aElement).css("backgroundColor",bkcolor);}

}

TextElement.prototype.updateText = function(iText, iTranslate){
	
	iText = iText.replace(/ /g,"&nbsp;"); 
	
	var aElement = $(this.element);
	
	this.text = iText; // in case of layout not inserted
	
	aElement.data("text",iText);
	if (valid(iTranslate)) {
		aElement.data("translate",iTranslate);
	}
	
	TextElement.changeLanguage(this.element);

	TextElement.resize(this.element);

}

/*
TextElement.linksCallback =  function(iElement) {
	return function() {
		TextElement.hashcall(iElement.element);
	};
}
*/

TextElement.prototype.afterInsert = function() {

	//console.log("after insert text element ");
	//console.log($(this.element));
	
	var aElement = $(this.element);
	
	aElement.disableSelection();
	
	if (!(typeof this.onClickFunction==="undefined")) {
		aElement.click(this.onClickFunction);
		aElement.css('cursor', 'pointer');
	}
	
	aElement.data("position",this.position);
	aElement.data("vposition",this.vposition);
	aElement.data("isurl",this.url);
	aElement.data("ishtmltext",this.htmltext);
	aElement.data("translate",this.translate);
	aElement.data("linked",this.linked);
	//console.log("Insert Text");
	//console.log(aElement);
	//console.log("Text value:"+this.text);
	aElement.data("text",this.text);
	//console.log("Text recorded:"+aElement.data("text"));
	aElement.data("linkcolor",this.linkColor);
	if (valid(this.color)) {
		aElement.data("color",this.color);
	} else {
		aElement.data("color",aElement.css("color"));
	}
	
	aElement.data("bkcolor",aElement.css("backgroundColor"));
	
	if (valid(this.onMouseOverColor) || valid(this.onBkMouseOverColor)) {
	
		//console.log("add mouse listener :"+this.onMouseOverColor);
	
		aElement.data("onMouseOverColor",this.onMouseOverColor);
		aElement.data("onBkMouseOverColor",this.onBkMouseOverColor);
		
		aElement.mouseenter(function(e){
			TextElement.onMouseEnterCB(aElement);
		});
		aElement.mouseout(function(e){
			TextElement.onMouseOutCB(aElement);
		});
	
	}
	
	TextElement.postInsertBuildLinks(this.context);
	
	TextElement.resize(this.element);
	
}

TextElement.resize = function(iElement,iChain,iForceResize) {

	//console.log("resize text element ");
	//console.log($(iElement));
	
	var textelement = $(iElement);
	var parentElement = textelement.parent();
	
	var elem = textelement.data("parentWrap");
	var wrap = (typeof elem==="undefined"?false:elem);

	//console.log('wrap:'+wrap);
	
	if (wrap) {
	
		textelement.css({ top : "0px" , left : "0px" });
		textelement.height( textelement.textHeight() );
		textelement.width( textelement.textWidth() );
	
	} else {
	
		var position = textelement.data("position");
		var vposition = textelement.data("vposition");
		var ishtmltext = textelement.data("ishtmltext");
		//console.log("Position="+position);
		//console.log("V Position="+vposition);
		
		var fontHeight = textelement.textHeight();
		var fontWidth = textelement.textWidth();
		textelement.height(fontHeight);
		textelement.width(fontWidth);
		
		//console.log("Width:"+fontWidth);
		//console.log("Height:"+fontHeight);
		
		var aWantedHeight = parentElement.height();
		var aWantedWidth = parentElement.width();
		
		var aLeft=0;
		if (aWantedWidth>fontWidth) {
			if (!(typeof position==="undefined")) {
				
				if (ishtmltext) {
				
					//console.log('html text');
					var aBorders = textelement.outerWidth(true)-textelement.innerWidth();
					//console.log("Border Size:"+aBorders);
					textelement.width(aWantedWidth-aBorders);
				
				} else {
				
					if (position=="center") {
						aLeft = (aWantedWidth-fontWidth)/2;
					} else if (position=="right") {
						aLeft = aWantedWidth-fontWidth;
					} else if (position=="left") {
						aLeft = 0;
					}
			
				}
				
			} 
			
		} else {
			textelement.width(aWantedWidth);
		} 
		
		var aTop=0;
		if (valid(vposition)) {
		
			if (vposition=='center') {
				aTop = ((aWantedHeight-fontHeight)/2);
			} else if (vposition=='bottom') {
				aTop = aWantedHeight-fontHeight;
			} else {
				aTop = 0;
			}
		
		} else {
			aTop = ((aWantedHeight-fontHeight)/2); // Centering text by default
		}

		textelement.css({ top : aTop , left : aLeft });
		
	}
	
	Element.resizeSpecialElements(iElement,iChain,iForceResize);
	
}// ######################## DEFINE ELEMENT TEXT INPUT ##################
var TextInputElement = function(iText,iType){
	Element.call(this);
	this.text = iText;
	this.type = iType;
	this.padding = 5;
} 
TextInputElement.prototype = new Element();
TextInputElement.prototype.constructor = TextInputElement;

TextInputElement.prototype.setFontSize = function (iSize){
	this.size = iSize;
	return this;
}
	
TextInputElement.prototype.setPosition = function (iPosition){
	this.position = iPosition;
	return this;
}

TextInputElement.prototype.setColor = function (iColor){
	this.color = iColor;
	return this;
}

TextInputElement.prototype.setPadding = function (iPadding){
	this.padding = iPadding;
	return this;
}

TextInputElement.prototype.setAutoComplete = function (iSourceFunction, iSelectFunction, iMinLength, iOptions) {
	this.sourceFunction = iSourceFunction;
	this.selectFunction = iSelectFunction;
	this.minLength = iMinLength;
	this.autocompleteOptions = iOptions;
	return this;
}

TextInputElement.prototype.setColorDescription = function (iColorDescription){
	this.colorDescription = iColorDescription;
	return this;
}


TextInputElement.prototype.setFollowing = function (iInputElement){
	this.following = iInputElement;
	return this;
}

TextInputElement.prototype.onValidation = function (iValidateFunction){
	this.validate = iValidateFunction;
	return this;
}

TextInputElement.prototype.setKeyPressedFunction = function (iFunction){
	this.keypressFunction = iFunction;
	return this;
}

TextInputElement.prototype.translate = function(iField){
	this.translate = iField;
	return this;
}

TextInputElement.prototype.implement = function () {
    this.id  = elementID;
	this.element = "#elem"+elementID;
	
	var html_code ="";
	html_code = "<form class='textinputelement' style='position:relative; margin:0px; padding:0px; border 0px; width:100%; height:100%;'><input type='text' name='none' style='border :0px; width:0px; height:0px; {display:none}'/>";
	
	// Create html input code to be doubled when password type
	var html_input_code ="";
	if ($.browser.msie) {	
		html_input_code = "<input class='tieInput ui-autocomplete-input ui-corner-all' style='position:absolute; border:0px; top:0px; left:0px; margin:0px; padding:0px; ";
	} else {
		html_input_code = "<input class='tieInput ui-autocomplete-input ui-corner-all' style='display:block; position:absolute; border:0px; top:0px; left:0px; margin:0px; padding:0px; ";
	}					
	
	if (valid(this.size)) {
		html_input_code += "font-size:"+this.size+";";
	}
	if (!(typeof this.position==="undefined")) {
		if (this.position=='center') {
			html_input_code += "text-align:center;"; // have to define the size
		} else if (this.position=='left') {
			html_input_code += "text-align:left;"; 
		} else if (this.position=='right') {
			html_input_code += "text-align:right;";
		}

	}
	
	if (!(typeof this.color==="undefined")) {
		html_input_code += "color : "+this.colorDescription+";";
	}
	
	var html_input_code_hidden="";
	if (this.type=="password") {
		
		html_input_code_hidden = html_input_code + "{display:none}'";
		html_input_code_hidden += " id='elem"+elementID+"_H' type='"+this.type+"'";
		html_input_code_hidden += ">";
	} 
		
	html_input_code += "'";
		
	if (!(typeof this.text==="undefined")) {
		var textValue = this.text;
		if (!(typeof this.translate==="undefined")) {
			textValue = International.translate( Layout.language , this.translate, this.text);
		}
		html_input_code += " value='" + textValue + "'";
	}
			
	html_input_code += " id='elem"+elementID+"' type='text'";
	html_input_code += ">"; 
	elementID++;
	
	html_code = html_code + html_input_code + html_input_code_hidden +"</form>";
	return html_code;
}

TextInputElement.changeLanguage = function(iElement) {

	var aElement = $(iElement);
	
	TextInputElement.reset(aElement);

}

TextInputElement.followingFunction = function(iEventObject){

   TextInputElement.defaultFunction(iEventObject);
   var aTarget = iEventObject.currentTarget;
   var aForm = $(aTarget).parent();
   var keyCode = iEventObject.keyCode || iEventObject.which; 
   
   if (( keyCode == 13 ) || (keyCode == 9)) {
     //console.log("Return key "+keyCode+" => handle follow");
	 //console.log(iEventObject);
	 
	 /*
	 var id = "#"+$(aTarget).attr('id');
	 if ( id.indexOf("_H")!=-1 ) {
		id = id.substring(0,id.indexOf("_H"));
		//console.log(id);
	 }
	 var aTarget = id;
	 */
	 
	 var following = aForm.data("following");
	 $(following.element).focus();
	 
   }

}

TextInputElement.validateFunction = function(iEventObject){
 
   	
   TextInputElement.defaultFunction(iEventObject);
   var aTarget = iEventObject.currentTarget;
   var aForm = $(aTarget).parent();
   var keyCode = iEventObject.keyCode || iEventObject.which; 
   
   if ( keyCode == 13 ) {
     
	    /*
		var id = "#"+$(aTarget).attr('id'); // Get the original id to retrieve data
		if ( id.indexOf("_H")!=-1 ) {
			id = id.substring(0,id.indexOf("_H"));
		}
		var aTarget = id;
		*/

		var validate = aForm.data("validate");	 
		validate();
	 
   }

}

TextInputElement.reset = function (iTarget) {

	var aTarget = iTarget;
	var aForm = $(iTarget).parent();
	$(aTarget).val("");
	// there we have to switch to the description mode
	var id = "#"+$(aTarget).attr('id');
	if ( id.indexOf("_H")!=-1 ) {
		id = id.substring(0,id.indexOf("_H"));
		//console.log(id);
	}
	aTarget = id;
	
	$(aTarget).val("");
	aForm.data("state","unmodified");
	var textcolor = aForm.data("colordescription");
	var type = aForm.data("type");
	
	if (type=="password") {
		
		$(aTarget+"_H").hide();
		$(aTarget+"_H").focus();
		$(aTarget).show();
	
	}
	
	$(aTarget).css({ "color" : textcolor});
	var translate = aForm.data("translate");
	var textValue = aForm.data("text");
	if (!(typeof translate==="undefined")) {
		textValue = International.translate( Layout.language , translate, textValue);
	}
	$(aTarget).val(textValue);
	$(aTarget).selectRange(0,0);

}

TextInputElement.defaultFunction = function(iEventObject){
	
	//console.log("Text input for element ");
	
	var aTarget = iEventObject.currentTarget;
	var keyCode = iEventObject.keyCode || iEventObject.which || iEventObject.charCode; 
	
	//console.log($(aTarget));
	var aForm = $(aTarget).parent();
	var state = aForm.data("state");
	//console.log("State "+state);
	//console.log("Shift:"+iEventObject.shiftKey);
	//console.log("Code:"+iEventObject.keyCode);
	if (!(iEventObject.shiftKey && (iEventObject.keyCode==16))) {
	
		var permitModification =  (keyCode != 13) && (keyCode != 8) && (keyCode != 5) && (keyCode != 9) ; 
		if (state=="unmodified" && permitModification) {
		
			// ++ change color
			var textcolor = aForm.data("color");
		
			var type = aForm.data("type");
			var height = aForm.data("height");
			$(aTarget).val("");
			if (type=="password") {
				// console.log("Type password  ...");
				// have to hide element and show hidden one this password symbols
				$("#"+id).hide();
				var id = $(aTarget).attr('id');
				$("#"+id+"_H").show();
				$("#"+id+"_H").focus();
				$("#"+id+"_H").css({ "color" : textcolor});
				$("#"+id+"_H").height(height);
				
				// here we switch to the hidden element, we have to forward the char
				//$("#"+id+"_H").val(String.fromCharCode(iEventObject.charCode));
				
				aTarget = "#"+id+"_H";
			}	
			aForm.data("state","modified");
			
			if (!(typeof textcolor==="undefined")) {
				$(aTarget).css({ "color" : textcolor});
			}
			
		} else {	
		
			//console.log("KeyCode:"+keyCode);
			var length = $(iEventObject.currentTarget).val().length;
			//console.log("Val:"+$(iEventObject.currentTarget).val());
			//console.log("Val length:"+length);
			if (
				(( keyCode == 8 ) && ( length==1 ))
				|| 
				(length==0)
				) {
				
				// size will be 0 or is zero
				// there we have to switch to the description mode
				TextInputElement.reset(aTarget);
				
				iEventObject.preventDefault(); // This leads in removing any update on restored description
				
			} 
		
		}
	} else {
	
		iEventObject.preventDefault()
	
	}
	
}

TextInputElement.onClickFunction = function(iClickedEvent){
	
	//console.log("Clicked input text "+iClickedEvent);
	
	var iClickedElement = iClickedEvent.currentTarget;
	var state = $(iClickedElement).parent().data("state");
	//console.log("State "+state);
	if (state=="unmodified") {
	
		//console.log("State unmodified ...");
	
		$(iClickedElement).focus();
		$(iClickedElement).selectRange(0,0);
		
		iClickedEvent.preventDefault();
	
	}
	
}

TextInputElement.prototype.setValue = function(iValue){

	var aForm = $(this.element).parent();
	var state = aForm.data("state");
	
	var type = aForm.data("type");
	var aTarget = this.element;
	var id = $(aTarget).attr('id');
	if (type=='password') {
		aTarget = "#"+id+"_H";
	}	
	
	if (iValue=="") {
		
		TextInputElement.reset(this.element);
	
	} else {
		if (state=="unmodified") {
		
			var textcolor = aForm.data("color");
			aForm.data("state","modified");
			$(aTarget).css({ "color" : textcolor});
			$(aTarget).val(iValue);
			
		} else {
		
			$(aTarget).val(iValue);
		
		}
	}
	
}

TextInputElement.prototype.getValue = function(iValue){
	var aForm = $(this.element).parent();
	var state = aForm.data("state");
	//console.log("State "+state);
	if (state=="unmodified") {
		return "";
	} else {
		var type = aForm.data("type");
		var aTarget = this.element;
		var id = $(aTarget).attr('id');
		if (type=='password') {
			aTarget = "#"+id+"_H";
		}	
		return $(aTarget).val();
	}
}

TextInputElement.prototype.afterInsert = function() {

	//console.log("after insert text input element ");
	//console.log($(this.element));

	$(this.element).click(TextInputElement.onClickFunction);
	if (this.type=="password") { $(this.element+"_H").click(TextInputElement.onClickFunction); }
	
	if (valid(this.keypressFunction)) {
	
		$(this.element).keydown(this.keypressFunction);
		if (this.type=="password") { $(this.element+"_H").keydown(TextInputElement.keypressFunction); }
	
	} else if (valid(this.following)) {
	
		$(this.element).keydown(TextInputElement.followingFunction);
		if (this.type=="password") { $(this.element+"_H").keydown(TextInputElement.followingFunction); }
	
	} else if (valid(this.validate)) {
	
		$(this.element).keydown(TextInputElement.validateFunction);
		if (this.type=="password") { $(this.element+"_H").keydown(TextInputElement.validateFunction); }
	
	} else {
	
		$(this.element).keydown(TextInputElement.defaultFunction);
		if (this.type=="password") { $(this.element+"_H").keydown(TextInputElement.defaultFunction); }
	
	}
	
	if (valid(this.sourceFunction) && (valid(this.minLength)) && (valid(this.selectFunction))) {
		
		$(this.element).autocomplete({
			source:	this.sourceFunction,
			minLength: this.minLength,
			select: this.selectFunction
		});
		
	}
	
	if (this.type=="password") {
		$(this.element+"_H").hide();
	}
	
	var aForm = $(this.element).parent();
	
	aForm.data("text",this.text);
	aForm.data("translate",this.translate);
	aForm.data("state","unmodified");
	aForm.data("type",this.type);
	aForm.data("size",this.size);
	aForm.data("padding",this.padding);
	aForm.data("following",this.following);
	aForm.data("validate",this.validate);
	aForm.data("color",this.color);
	aForm.data("colordescription",this.colorDescription);
	
	TextInputElement.resize(aForm);
	
}

// TODO Add resize for textelement

TextInputElement.resize = function(iElement,iChain,iForceResize) {

	//console.log('wrap:'+wrap);
	
	var textelement = $(iElement); // Form
	var aParent = textelement.parent(); 
	
	var size = parseInt(textelement.css("font-size"), 10);
	var padding = textelement.data("padding");
	
	var height = 2*padding+size;
	var aPadding = padding;
	
	textelement.data("height",height);
	
	// padding: 11px 0px 11px 0px;
	
	//console.log(aParent);
	//console.log(textelement);
	//console.log("Width:"+aParent.width());
	//console.log("Height:"+aParent.height());
	
	textelement.width(aParent.width()); // Always parent width
	textelement.height(height);
	
	$.each( textelement.children(".tieInput") , function(key,value) {
		
		$(value).width(aParent.width());
		$(value).height(height);
		
		if ($.browser.msie) {
			var lineHeight = aPadding+size;
			$(value).css( "line-height" , height+"px");
			$(value).css({ "padding-top" : aPadding+"px", "padding-bottom" : aPadding+"px"});
		} else {
			$(value).css({ "padding-top" : aPadding+"px", "padding-bottom" : aPadding+"px"});
		}
		
	});
		
	Element.resizeSpecialElements(iElement,iChain,iForceResize);
}

// ######################## DEFINE ELEMENT BUTTON ######################
var ButtonElement = function(iText, iCallback){
	Element.call(this);
	this.text = iText;
	this.callback = iCallback;
} 
ButtonElement.prototype = new Element();
ButtonElement.prototype.constructor = ButtonElement;

ButtonElement.prototype.implement = function () {
	this.id  = elementID;
	this.element = "#elem"+elementID;
	var html_code = "<button id='elem"+elementID+"' class='buttonelement' >"+this.text+"</button>";
	elementID++;
	return html_code;
}

ButtonElement.prototype.afterInsert = function() {
	//console.log("after insert Button");
	//console.log($(this.element).find("button"));
	$(this.element).button().click(this.callback); // ui-jquery
}

// TODO implement resize// ######################## DEFINE EMPTY ELEMENT  ########################
var EmptyElement = function(){
	Element.call(this);
} 
EmptyElement.prototype = new Element();
EmptyElement.prototype.constructor = EmptyElement;

EmptyElement.prototype.implement = function () {
	this.id = elementID;
	var html_code = "<div id='elem"+elementID+"' style='position:absolute; border:0px; top:0px; left:0px; margin:0px; padding:0px;width:100%;height:100%;' ></div>";
	elementID++;
	return html_code;
}

EmptyElement.prototype.afterInsert = function() {
	//console.log("after insert empty element");
}// ######################## DEFINE Container Element ######################

var ContainerElement = function(iElement){
	Element.call(this);
	this.id = null;
	this.internalElement = iElement;
} 

ContainerElement.prototype = new Element();
ContainerElement.prototype.constructor = ContainerElement;

ContainerElement.prototype.addStyle = function (iStyle) {
	this.style = iStyle;
	return this;
}

ContainerElement.prototype.addClass = function (iClass) {
	this.addedClass = iClass;
	return this;
}

ContainerElement.prototype.getInnerElements = function () {
	return [this.internalElement];
}

ContainerElement.prototype.onClick = function (iOnClickFunction){
	this.onClickFunction = iOnClickFunction;
	return this;
}

ContainerElement.prototype.onEnterBkColor = function(iBkColor){
	this.onEnterBkColorV = iBkColor;
}

ContainerElement.prototype.implement = function () {
	this.id = elementID;
	this.element =  "#elem"+this.id;
	//this.internalElement.element = "#elem"+this.id; 
	elementID++;
	
	// Here test wrapping definition
	this.internalElement = this.checkWrapping(this.internalElement);
	
	var html_code =  "<div id='elem"+this.id+"' class='containerElement ui-widget-content ui-corner-all";
	
	if (!(typeof this.addedClass==='undefined')) {
		html_code = html_code + this.addedClass+"'";
	} else {
		html_code = html_code + "'";
	}
	
	html_code = html_code + "style='position: relative; overflow:hidden; width :100%; height : 100%;";
	
	if (!(typeof this.style==='undefined')) {
		html_code = html_code + this.style+"'>";
	} else {
		html_code = html_code + "'>";
	}
	
	html_code = html_code +this.internalElement.implement()+"</div>";
	
	return html_code;
}

ContainerElement.onMouseEnterCB = function(aElement){

	//console.log("Mouse Enter");
	var bkcolor = $(aElement).data('onEnterBkColor');
	if (valid(bkcolor)) {
		$(aElement).css("backgroundColor",bkcolor);
		$(aElement).css("backgroundImage", "none");
	}

}

ContainerElement.onMouseOutCB = function(aElement){

	//console.log("Mouse Out");
	var bkcolor = $(aElement).data('bkcolor');
	if (valid(bkcolor)) {$(aElement).css("backgroundColor",bkcolor);}

}

ContainerElement.prototype.afterInsert = function() {
	
	//console.log("after insert container");
	//console.log(this.element);
	
	var aElement = $(this.element);
	
	aElement.data('prop', this.prop);
	
	if (valid(this.attached)) {
		$(aElement).data("attached",this.attached);
	}
	
	if (valid(this.onClickFunction)) {
		console.log("On click function installed");
		aElement.click(this.onClickFunction);
		aElement.css('cursor', 'pointer');
	}
	
	aElement.data("bkcolor",aElement.css("backgroundColor"));
	if (valid(this.onEnterBkColorV)) {
	
		//console.log("add mouse listener :"+this.onMouseOverColor);
	
		aElement.data("onEnterBkColor",this.onEnterBkColorV);
		
		aElement.mouseover(function(e){
			ContainerElement.onMouseEnterCB(aElement);
		});
		aElement.mouseout(function(e){
			ContainerElement.onMouseOutCB(aElement);
		});
	
	}
	
	if (!(typeof this.internalElement === "undefined")) {
	
		var wrap = false;
		if (!(typeof this.prop==='undefined')) {
		
			if (!(typeof this.prop.wrap==='undefined')) {
				
				wrap = this.prop.wrap;
				
			}
		
		}
	
		//console.log('wrap container:'+wrap);
		$(this.internalElement.element).data('parentWrap',wrap);
	
		this.internalElement.afterInsert();		
		
	} 
	
	ContainerElement.resize(this.element);
	
}

// Static method
ContainerElement.resize = function (iContainer,iChain, iForceResize){
	
	//console.log("Container");
	//console.log(iContainer);
	var aContainer = $(iContainer);
	var prop = aContainer.data('prop');
	
	//console.log(prop);
	var wrap = false;
	var fixed = false;
	if (!(typeof prop==='undefined')) {
	
		if (!(typeof prop.wrap==='undefined')) {
			
			wrap = prop.wrap;
			
		}
		
		if (valid(prop.fixed)) {
			fixed = true;
		}
	
	}
	
	if (wrap && !fixed) { // wrap size of inner element
	
		handleDependentResize(aContainer,iChain,iForceResize); // Pre resize
		var children = aContainer.children();
		aContainer.width($(children).outerWidth(true));
		aContainer.height($(children).outerHeight(true));
		
	} else if (fixed) { // Fixed means container have fixed size
		
		aContainer.width(prop.width);
		aContainer.height(prop.height);
		handleDependentResize(aContainer,iChain,iForceResize);
		
	} else { // Default mode takes the size of parents elements
		var parent = aContainer.parent();
		
		var hBorder = aContainer.outerWidth()-aContainer.innerWidth();
		var vBorder = aContainer.outerHeight()-aContainer.innerHeight();
		
		aContainer.width(parent.innerWidth()-(hBorder+parseInt(parent.css("padding-left"))+parseInt(parent.css("padding-right")))); // TODO correct this with inner/outer config 
		aContainer.height(parent.innerHeight()-(vBorder+parseInt(parent.css("padding-top"))+parseInt(parent.css("padding-bottom"))));
		handleDependentResize(iContainer,iChain,iForceResize);
	}	
	
	Element.resizeSpecialElements(iContainer,iChain,iForceResize);
	
}
// ######################## DEFINE Mapper Element ######################

var MapperElement = function(iID){
	Element.call(this);
	this.id = iID;
	this.element = "#"+iID;
} 

MapperElement.prototype = new Element();
MapperElement.prototype.constructor = MapperElement;


MapperElement.prototype.implement = function () {
	
	var html_code =  "<div id='"+this.id+"' class='mapperelement' style='position:absolute; top 0px; left: 0px; margin : 0px; padding: 0px; overflow : visible;' ></div>";
	return html_code;
	
}

MapperElement.prototype.afterInsert = function() {
	
	$(this.element).data('prop', this.prop);
	
	MapperElement.resize(this.element);
	
}

// Static method
MapperElement.resize = function(iMapper,iChain,iForceResize){
	
	//console.log("Resize mapper :"+iMapper);
	//console.log($(iMapper).attr('id'));
	
	var aMapper = $(iMapper);
	//console.log(prop);
	var wrap = false;
	if (!(typeof prop==='undefined')) {
	
		if (!(typeof prop.wrap==='undefined')) {
			
			wrap = prop.wrap;
			
		}
	
	}
	
	if (wrap) {
		
		handleDependentResize(aMapper,iChain,iForceResize); // Pre resize
		var children = aMapper.children();
		if (!(typeof children==="undefined")) {
			aMapper.width(children.outerWidth(true));
			aMapper.height(children.outerHeight(true));
		} else {
			aMapper.width(0);
			aMapper.height(0);
		}
		
	} else {
		var parent = aMapper.parent();
		//console.log("width:"+parent.innerWidth());
		//console.log("height:"+parent.innerHeight());
		aMapper.width(parent.innerWidth()); 
		aMapper.height(parent.innerHeight());
		handleDependentResize(aMapper,iChain,iForceResize);
	}	
	
	Element.resizeSpecialElements(iMapper,iChain);
	
}
// ######################## DEFINE Container Element Scrollbar ###############

var ScrollContainerElement = function(iElement){
	Element.call(this);
	this.id = null;
	this.internalElement = iElement;
	
} 

ScrollContainerElement.prototype = new Element();
ScrollContainerElement.prototype.constructor = ScrollContainerElement;

ScrollContainerElement.prototype.setStyle = function(iStyle){
	this.style = iStyle;
}

ScrollContainerElement.prototype.setCanvasStyle = function(iCanvasStyle){
	this.cstyle = iCanvasStyle;
}

ScrollContainerElement.prototype.implement = function () {

	this.id = elementID;
	//this.internalElement.element = "#elem"+this.id; 
	this.element = "#elem"+this.id; 
	elementID++;
	
	// Here test wrapping definition
	this.internalElement = this.checkWrapping(this.internalElement);

	var html_code = "<div  id='elem"+this.id+"' class='scrollContainerElement ui-widget-content ui-corner-all' style='position: relative; overflow: hidden;"+(valid(this.style)?this.style:"")+"'>"+
				"<div class='scrollPanelElement' style='position: relative; "+(valid(this.cstyle)?this.cstyle:"")+" '>"+
					"<div  class='scrollCanvas' style='position:absolute; left: 0px; top: 0px; width:0px; height:0px;'>"+
						this.internalElement.implement()+
					"</div>"+
				"</div>"+
				"<div class='verticalScrollBar ui-widget-content ui-corner-right' style='position : absolute; top : 0px; left: 0px;' >"+
					"<div class='scrollbutton ui-widget-content ui-corner-all' style='position : absolute; top : 0px; left: 0px;' ></div>"+
				"</div>"+
				"<div class='horizontalScrollBar ui-widget-content ui-corner-bottom' style='position : absolute; top : 0px; left: 0px;'>"+
					"<div class='scrollbutton ui-widget-content ui-corner-all' style='position : absolute; top : 0px; left: 0px;' ></div>"+
				"</div>"+
			"</div>";
	return html_code;
}

ScrollContainerElement.prototype.getInnerElements = function () {
	return [this.internalElement];
}

var K_SIZE = 20;

ScrollContainerElement.verticalPlacement = function(iContainer){
	//var aContainer = $(ui.handle).closest(".scrollContainerElement").parent();
	var aRecord = $(iContainer).data('prop');
	var aCanvas = $(iContainer).find(".scrollCanvas");
	var vPercent = $(iContainer).data('vPercent');
	
	//console.log(aRecord);
	
	if (!(typeof aRecord==='undefined')) {
	
		//console.log(aRecord.vBar);
		if ( aRecord.vBar ) {
			
			var aYPosition = (vPercent*(aRecord.vBarCSize-$(aCanvas).height()))/100;
			
			//console.log("Y position :"+aYPosition);
			//console.log((aCanvas));
			$(aCanvas).css({ top : aYPosition+"px" });
			
		}
		
	}
}

ScrollContainerElement.horizontalPlacement = function(iContainer){
	
	var aRecord = $(iContainer).data('prop');
	var aCanvas = $(iContainer).find(".scrollCanvas");
	var hPercent = $(iContainer).data('hPercent');
	
	//console.log(aRecord);
	
	if (!(typeof aRecord==='undefined')) {
	
		//console.log(aRecord.hBar);
		if ( aRecord.hBar ) {
			
			var aXPosition = (hPercent*(aRecord.hBarCSize-$(aCanvas).width()))/100;
			
			//console.log("X position :"+aXPosition);
			//console.log(aCanvas);
			$(aCanvas).css({ left : aXPosition+"px" });
			
		}
		
	}
}

// Vertical Management
ScrollContainerElement.verticalButtonMouseDown = function(iEvent,iElement) {

	//console.log("Mouse down event on verticalButton");
	//console.log(iElement);
	
	$(iElement).data('vActivated',true);
	$(iElement).data('vMPos',iEvent.pageY);
	
}

ScrollContainerElement.verticalButtonMouseUp = function(iEvent,iElement) {
	
	//console.log("Mouse up event on verticalButton");
	//console.log(iElement);
	
	$(iElement).data('vActivated',false);
	
}

ScrollContainerElement.verticalButtonMouseMove = function(iEvent,iElement) {

	var aActivated = $(iElement).data('vActivated');
	if (aActivated) {
	
		var vMousePos = iEvent.pageY;
		var vPrecedingPos = $(iElement).data('vMPos');
		var vmove = vMousePos-vPrecedingPos;
		
		var aButton = $(iElement).find( ".verticalScrollBar" ).find(".scrollbutton");
		var aTop = parseInt(aButton.css('top'),10);
		aTop = aTop + vmove;
		
		var aVMax = $(iElement).data('vMax');
		
		if (aTop<0) {
		  aTop = 0;
		} else if (aTop>=aVMax) {
		  aTop = aVMax;	
		}
		
		aButton.css({top:aTop});
		
		var aPercent = aTop*100/aVMax;
		//console.log("VPos:"+aPercent+"%");
		$(iElement).data('vPercent', aPercent);
		$(iElement).data('vMPos',vMousePos);
	
		ScrollContainerElement.verticalPlacement($(iElement));
	
	}

}

ScrollContainerElement.verticalButtonMouseEnter = function(iEvent,iElement) {

	//console.log("Mouse enter event on verticalButton");
	//console.log(iEvent);
	
	if (iEvent.buttons==0) {
		$(iElement).data('vActivated',false);
	}
	
}

ScrollContainerElement.verticalBarClick = function(iEvent,iElement) {

	//console.log("Vertical bar click :");
	
	//var aVerticalBar = $(iElement).find( ".verticalScrollBar" );
	var aButton = $(iElement).find( ".verticalScrollBar" ).find(".scrollbutton");
	var aButtonPos = $(aButton).offset().top;
	
	var aMax = $(iElement).data('vMax');
	var vPercent = $(iElement).data('vPercent');
	
	if (aButtonPos>iEvent.pageY) {
		
		//console.log("top");
		
		vPercent = vPercent - 5;
		if (vPercent<0) {
			vPercent = 0;
		}
		
		
	} else {
		
		//console.log("bottom");
		
		vPercent = vPercent + 5;
		if (vPercent>100) {
			vPercent = 100;
		}
		
	}
	
	var aTop = vPercent*aMax/100;
	$(aButton).css({top:aTop+'px'});
	$(iElement).data('vPercent',vPercent);
	
	ScrollContainerElement.verticalPlacement($(iElement));
	
}

ScrollContainerElement.horizontalButtonMouseDown = function(iEvent,iElement) {

	//console.log("Mouse down event on horizontalButton");
	//console.log(iElement);
	
	$(iElement).data('hActivated',true);
	$(iElement).data('hMPos',iEvent.pageX);
	
}

ScrollContainerElement.horizontalButtonMouseUp = function(iEvent,iElement) {
	
	//console.log("Mouse up event on horizontalButton");
	//console.log(iElement);
	
	$(iElement).data('hActivated',false);
	
}

ScrollContainerElement.horizontalButtonMouseMove = function(iEvent,iElement) {

	var aActivated = $(iElement).data('hActivated');
	if (aActivated) {
	
		var hMousePos = iEvent.pageX;
		var hPrecedingPos = $(iElement).data('hMPos');
		var hmove = hMousePos-hPrecedingPos;
		
		var aButton = $(iElement).find( ".horizontalScrollBar" ).find(".scrollbutton");
		var aLeft = parseInt(aButton.css('left'),10);
		aLeft = aLeft + hmove;
		
		var aHMax = $(iElement).data('hMax');
		
		if (aLeft<0) {
		  aLeft = 0;
		} else if (aLeft>=aHMax) {
		  aLeft = aHMax;	
		}
		
		aButton.css({left:aLeft});
		
		var aPercent = aLeft*100/aHMax;
		//console.log("VPos:"+aPercent+"%");
		$(iElement).data('hPercent', aPercent);
		$(iElement).data('hMPos',hMousePos);
		
		ScrollContainerElement.horizontalPlacement($(iElement));
		
	}

}

ScrollContainerElement.horizontalButtonMouseEnter = function(iEvent,iElement) {

	//console.log("Mouse enter event on verticalButton");
	//console.log(iEvent);
	
	if (iEvent.buttons==0) {
		$(iElement).data('hActivated',false);
	}
	
}

ScrollContainerElement.horizontalBarClick = function(iEvent,iElement) {

	//console.log("Horizontal bar click :");
	
	var aButton = $(iElement).find( ".horizontalScrollBar" ).find(".scrollbutton");
	var aButtonLeft = $(aButton).offset().left;
	
	var aMax = $(iElement).data('hMax');
	var hPercent = $(iElement).data('hPercent');
	
	if (aButtonLeft>iEvent.pageX) {
		
		//console.log("left");
		
		hPercent = hPercent - 5;
		if (hPercent<0) {
			hPercent = 0;
		}
		
		
	} else {
		
		//console.log("right");
		
		hPercent = hPercent + 5;
		if (hPercent>100) {
			hPercent = 100;
		}
		
	}
	
	var aLeft = hPercent*aMax/100;
	$(aButton).css({left:aLeft+'px'});
	$(iElement).data('hPercent',hPercent);
	
	ScrollContainerElement.horizontalPlacement($(iElement));
}

ScrollContainerElement.prototype.afterInsert = function() {

	//console.log("after insert ScrollContainerElement");
	
	// Vertical Scroll Bar
	var aElement = $(this.element);
	
	aElement.data('vActivated',false);
	aElement.data('vPercent',0);
	aElement.data('hActivated',false);
	aElement.data('hPercent',0);
	
	var vscrollbar = aElement.find( ".verticalScrollBar" );
	var vButton = vscrollbar.find( ".scrollbutton" );
	vscrollbar.click(function(e){
		e.preventDefault();
		e.stopPropagation();
		ScrollContainerElement.verticalBarClick(e,aElement);
	});
	
	vButton.append( "<span class='ui-icon ui-icon-grip-dotted-horizontal'></span>" );
	vButton.mousedown(function(e){
		e.preventDefault();
		e.stopPropagation();
		ScrollContainerElement.verticalButtonMouseDown(e,aElement);
	});
	vButton.mouseenter(function(e){
		e.preventDefault();
		e.stopPropagation();
		ScrollContainerElement.verticalButtonMouseEnter(e,aElement);
	});
	vButton.mousemove(function(e){
		e.preventDefault();
		e.stopPropagation();
		ScrollContainerElement.verticalButtonMouseMove(e,aElement);
	});
	vButton.mouseup(function(e){
		e.preventDefault();
		e.stopPropagation();
		ScrollContainerElement.verticalButtonMouseUp(e,aElement);
	});
	
	// Horizontal Scroll Bar
	var hscrollbar = aElement.find( ".horizontalScrollBar" );
	var hButton = hscrollbar.find( ".scrollbutton" );
	hscrollbar.click(function(e){
		e.preventDefault();
		e.stopPropagation();
		ScrollContainerElement.horizontalBarClick(e,aElement);
	});
	
	hButton.append( "<span class='ui-icon ui-icon-grip-dotted-vertical'></span>" );
	hButton.mousedown(function(e){
		e.preventDefault();
		e.stopPropagation();
		ScrollContainerElement.horizontalButtonMouseDown(e,aElement);
	});
	hButton.mouseenter(function(e){
		e.preventDefault();
		e.stopPropagation();
		ScrollContainerElement.horizontalButtonMouseEnter(e,aElement);
	});
	hButton.mousemove(function(e){
		e.preventDefault();
		e.stopPropagation();
		ScrollContainerElement.horizontalButtonMouseMove(e,aElement);
	});
	hButton.mouseup(function(e){
		e.preventDefault();
		e.stopPropagation();
		ScrollContainerElement.horizontalButtonMouseUp(e,aElement);
	});
	
	$(this.internalElement.element).data("parentWrap",true); // The inner element have are wrapped by parents
	this.internalElement.afterInsert();
	
	ScrollContainerElement.resize(aElement);
	
}

// Static method
ScrollContainerElement.resize = function (iContainer,iChain,iForceResize){
	
	//console.log("Resize Scroll Container");
	//console.log(iContainer);
	
	var aContainer = $(iContainer);
	
	// Define container size
	var aParent = aContainer.parent();
	
	var hBorder = aContainer.outerWidth()-aContainer.innerWidth();
	var vBorder = aContainer.outerHeight()-aContainer.innerHeight();
	
	var innerWidth = aParent.innerWidth()-(hBorder+parseInt(aParent.css("padding-left"))+parseInt(aParent.css("padding-right")));
	var innerHeight = aParent.innerHeight()-(vBorder+parseInt(aParent.css("padding-top"))+parseInt(aParent.css("padding-bottom")));
	
	aContainer.width(innerWidth);
	aContainer.height(innerHeight);
	
	// Define scroll panel size
	var aScrollPanel = aContainer.children(".scrollPanelElement");
	//console.log(aScrollPanel);
	aScrollPanel.width(innerWidth);
	aScrollPanel.height(innerHeight);

	// Find elements Canvas, VScrollBar , HScrollBar
	var aScrollCanvas = aScrollPanel.children(".scrollCanvas");
	var aVScrollBar = aContainer.children(".verticalScrollBar");
	var aHScrollBar = aContainer.children(".horizontalScrollBar");
	
	// Resize contained element
	//console.log("Resizing the content:");
	//console.log(aScrollCanvas);
	handleDependentResize(aScrollCanvas,iChain,iForceResize);
	
	// Resize canvas
	var children = aScrollCanvas.children();
	//console.log(children);
	aScrollCanvas.width(children.outerWidth());
	aScrollCanvas.height(children.outerHeight());
	
	var vContWidth = aContainer.innerWidth();
	var vContHeight = aContainer.innerHeight();
	
	var cWidthSize = aScrollCanvas.width();
	var cHeightSize = aScrollCanvas.height();
	
	var vScrollBar = false;
	var hScrollBar = false;
	
	// Test Width/Height
	if ( cHeightSize > vContHeight) {
		vScrollBar = true;
		vContWidth = vContWidth-K_SIZE;
	}
	
	if ( cWidthSize > vContWidth ) {
		hScrollBar = true;
		vContHeight = vContHeight-K_SIZE;
	}
	
	if ((!vScrollBar) && (cHeightSize > vContHeight)) {
		vScrollBar = true;
		vContWidth = vContWidth-K_SIZE;
	}
	
	// three times to detect all cases
	
	if (vScrollBar) {
	
		aVScrollBar.show();
		aVScrollBar.css({ top: "0px", left : (vContWidth+2)+"px" });
		aVScrollBar.width(K_SIZE);
		aVScrollBar.height(vContHeight+1);
		
		var cursorSize =  (1-(cHeightSize - vContHeight)/cHeightSize)*vContHeight;
		if (cursorSize<16) {
			cursorSize = 16;
		}
		//console.log("Cursor height size:"+cursorSize);
		var aButton = aVScrollBar.find( ".scrollbutton");
		aButton.height(cursorSize);
		
		var hBorder = aButton.outerWidth()-aButton.innerWidth();
		var innerWidth = K_SIZE-(hBorder*2);
		aButton.width(innerWidth);
		
		//aElement.data('vActivated',false);
		var vBorder = aButton.outerHeight()-aButton.innerHeight();
		var aMax = vContHeight-(aButton.outerHeight()+vBorder);
		
		var vPercent = aContainer.data('vPercent');
		var aTop = vPercent*aMax/100;
		aButton.css({top:aTop+'px'});
		
		aContainer.data('vMax',aMax);
		
	} else {
	
		aVScrollBar.hide();
	
	}
	
	if (hScrollBar) {
		
		aHScrollBar.show();
		aHScrollBar.css({ top: (vContHeight+2)+"px", left : "0px" });
		aHScrollBar.width(vContWidth+1);
		aHScrollBar.height(K_SIZE);
		
		var cursorWidthSize =  (1-(cWidthSize - vContWidth)/cWidthSize)*vContWidth;
		if (cursorWidthSize<16) {
			cursorWidthSize = 16;
		}
		//console.log("Cursor width size:"+cursorWidthSize);
		var aButton = aHScrollBar.find( ".scrollbutton");
		aButton.width(cursorWidthSize);
		
		var vBorder = aButton.outerHeight()-aButton.innerHeight();
		var innerHeight = K_SIZE-(vBorder*2);
		aButton.height(innerHeight);
		
		var hBorder = aButton.outerWidth()-aButton.innerWidth();
		var aMax = vContWidth-(aButton.outerWidth()+hBorder);
		
		var hPercent = aContainer.data('hPercent');
		var aLeft = hPercent*aMax/100;
		aButton.css({left:aLeft+'px'});
		
		aContainer.data('hMax',aMax);
		
	} else {
		
		aHScrollBar.hide();
		
	}
	
	aContainer.data('prop', { hBar : hScrollBar, vBar : vScrollBar, vBarCSize : vContHeight , hBarCSize : vContWidth } );
		
}// ######################## DEFINE ELEMENT TEXT ########################
var VerticalBarElement = function(){
	Element.call(this);
} 
VerticalBarElement.prototype = new Element();
VerticalBarElement.prototype.constructor = VerticalBarElement;

VerticalBarElement.prototype.setPosition = function (iPosition){
	this.position = iPosition;
	return this;
}

VerticalBarElement.prototype.setStyle = function (iStyle){
	this.style = iStyle;
	return this;
}

VerticalBarElement.prototype.setVStyle = function (iStyle){
	this.vstyle = iStyle;
	return this;
}

VerticalBarElement.prototype.setColor = function (iColor){
	this.color = iColor;
	return this;
}

VerticalBarElement.prototype.setWidth = function (iWidth){
	this.width = iWidth;
	return this;
}

VerticalBarElement.prototype.setHeight = function (iHeight){
	this.height = iHeight;
	return this;
}


VerticalBarElement.prototype.implement = function () {
    this.id  = elementID;
	this.element = "#elem"+elementID;
	var html_code = "<div id='elem"+elementID+"' class='verticalbarelement' style='position:absolute; top:0px; left:0px;  ";
	elementID++;

	if (valid(this.vstyle)) {
		html_code += this.vstyle;
	}

	if (!(typeof this.color==="undefined")) {
		html_code = html_code + "border-left-color: "+this.color+";";
	} else {
		html_code = html_code + "border-left-color: #000000;";
	}
	
	if (!(typeof this.style==="undefined")) {
		html_code = html_code + "border-left-style: "+this.style+";";
	} else {
		html_code = html_code + "border-left-style: solid;";
	}
	
	if (!(typeof this.width==="undefined")) {
		html_code = html_code + "border-left-width: "+this.width+";";
	} else {
		html_code = html_code + "border-left-width: 1px;";
		this.width = 1;
	}
	
	html_code = html_code + "'></div>";
	
	return html_code;
}


VerticalBarElement.prototype.afterInsert = function() {

	$(this.element).data("position",this.position);
	$(this.element).data("color",this.color);
	$(this.element).data("style",this.style);
	$(this.element).data("width",this.width);
	if (valid(this.height)) {
		$(this.element).data("height",this.height);
	} else {
		$(this.element).data("height",100);
	}
	
	VerticalBarElement.resize(this.element);
	
}

VerticalBarElement.resize = function(iElement,iChain,iForceResize) {
	
	var verticalElement = $(iElement);
	var parentElement = $(iElement).parent();
	
	var elem = verticalElement.data("parentWrap");
	var wrap = (typeof elem==="undefined"?false:elem);

	var height = verticalElement.data("height");
	
	
	if (wrap) {
	
		verticalElement.css({ top : "0px" , left : "0px" });
		verticalElement.height(height);
	
	} else { 
	
		//console.log("No wrap");
	
		var position = verticalElement.data("position");
		var width = verticalElement.data("width");
		
		var aIHeight = parentElement.innerHeight();
		var aWantedHeight = aIHeight*height/100;
		var aWantedWidth = parentElement.innerWidth();
		
		var aTop = (aIHeight-aWantedHeight)/2;
		verticalElement.height(aWantedHeight);
		
		var aLeft=0;
		if (!(typeof position==="undefined")) {
			
			if (position=="center") {
				aLeft = (aWantedWidth)/2;
			} else if (position=="right") {
				aLeft = aWantedWidth-width;
			} else {
				aLeft = width;
			}
			
		}

		verticalElement.css({ top : aTop+"px" , left : aLeft+"px" });
		
	}
	
	Element.resizeSpecialElements(iElement,iChain,iForceResize);
	
}// ######################## DEFINE ELEMENT SLIDER ######################
var SliderElement = function(iValue, iRange, iCallback){
	Element.call(this);
	this.value = iValue;
	this.range = iRange;
	this.callback = iCallback;
} 
SliderElement.prototype = new Element();
SliderElement.prototype.constructor = SliderElement;

SliderElement.prototype.setSize = function (iWidth,iHeight){
	this.width = iWidth;
	this.height = iHeight;
}

SliderElement.prototype.setRange = function (iRange){
	this.range = iRange;
}

SliderElement.prototype.setBackgroundColor = function (iColor){
	this.backgroundColor = iColor;
}

SliderElement.prototype.implement = function () {
	this.id  = elementID;
	this.element = "#elem"+elementID;
	var html_code = "<div id='elem"+elementID+"' class='sliderelement' ><div class='inner'></div></div>";
	elementID++;
	return html_code;
}

SliderElement.prototype.afterInsert = function() {

	var aRange = "min";
	if (valid(this.range)) {
		aRange = this.range;
	}

	$(this.element).find('.inner').slider({
		value: this.value,
		values: this.value,
		orientation: "horizontal",
		range: aRange,
		min : this.range.min,
		max : this.range.max,
		slide: this.callback
	});
	
	if (valid(this.backgroundColor)) {
		$(this.element).css("backgroundColor",this.backgroundColor);
		$(this.element).find(".inner").css({ "backgroundColor" : this.backgroundColor,
											 "backgroundImage" : "none" } );
	}
	$(this.element).find(".inner").css({"border": "0px"});
	
	$(this.element).data('width',this.width);
	$(this.element).data('height',this.height);
	$(this.element).data('value',this.value);
	$(this.element).data('min',this.range.min);
	$(this.element).data('max',this.range.max);
	$(this.element).data('callback',this.callback);
}

SliderElement.resize = function (iElement,iChain,iForceResize){
	
	var aElement = $(iElement);
	var aInner = $(iElement).find(".inner");
	var aHandle = $(iElement).find(".inner").find("a");
	var aSlider = $(iElement).find(".inner").find("div");
	
	var wrap = (!(typeof prop==="undefined")?prop.wrap:false);
	if (!wrap) {
		var elem = aElement.data("parentWrap");
		wrap = (typeof elem==="undefined"?false:elem);
	}
	
	var width = 0;
	var height = 0;
	if (!wrap) {
		var aParent = aElement.parent();
		width = aParent.width();
		height = aParent.height();
	} else {
		width = aElement.data("width");
		height = aElement.data("height");
	}
	
	aElement.width(width);
	aElement.height(height);
	
	aInner.width(width-height);
	aInner.height(height);
	aInner.css("left","0px");
	
	// margin-left margin-right
	aSlider.height(height);
	
	aHandle.css({"padding" : "0px"});
	aHandle.css({"margin-left" : "0px" });
	aHandle.width(height-4);
	aHandle.height(height-4);
	
	var aTop = (aElement.height()-aHandle.outerHeight())/2;
	aHandle.css("top", aTop+"px");
	
	Element.resizeSpecialElements(iElement,iChain,iForceResize);
	
}

// TODO implement resize// ######################## DEFINE LAYOUT     ##########################
var Layout = function(){
	Element.call(this);
	this.prop=null;
} 
Layout.prototype = new Element();
Layout.prototype.constructor = Layout;

window.recordedFunction = [];

Layout.recordFunction = function(iName,iFunction){
	window.recordedFunction[iName]=iFunction;
}

Layout.prototype.afterInsert = function (){
	//console.log("Default after insert");
}

Layout.prototype.resize = function(){
	//console.log("Default resize");
}

Layout.language = "en";

Layout.setLanguage = function(iLanguage){
	
	Layout.language = iLanguage;
	
	var aDocument = $(document);
	// Here we have to find each item that uses languages
	$.each( aDocument.find(".textelement"), function(key,value) {
		if ($.contains(document.documentElement, $(value)[0])) { // by changing the language a TextElement can disappear
			TextElement.changeLanguage(value);
		}
	});
	
	var aHash = buildLocationHash(getLocationSplit());
	TextElement.hashcall(aHash); // Rebuild pages
	
	$.each( aDocument.find(".textinputelement"), function(key,value) {
		TextInputElement.changeLanguage(value);
	});
	
	handleDependentResize($(document),"",true);
	
}




// ######################## DEFINE GRIDLAYOUT ##########################

var GridLayout = function(iWidth,iHeight){
	Layout.call(this);
	this.width = iWidth;
	this.height = iHeight;
	this.columnsProperties = new Array(iWidth);
	this.container = new Array(iWidth);
	for (var i = 0; i < this.width; i++) {
		this.container[i] = new Array(this.height);
	}
}
GridLayout.prototype = new Layout();
GridLayout.prototype.constructor = GridLayout;

GridLayout.prototype.implement = function () {
	
	this.id  = elementID;
	this.element = "#elem"+elementID;
	elementID++;
	
	var html_code = "";
	if (this.prop!=null){
		
		html_code="<div id='elem"+this.id+"' class='gridlayout' "; // identify class layout to handle resize
		
		//if (!(typeof this.prop.id==="undefined")) {html_code = html_code + " id='"+this.prop.id+"' ";}
		if (!(typeof this.prop.classtype==="undefined")) {html_code = html_code + " class='"+this.prop.classtype+"' ";}
		
		//Style
		html_code = html_code + " style='";
		if (!(typeof  this.prop.fullscreen==="undefined")) {
			if ( this.prop.fullscreen) {
				html_code = html_code + "position:fixed; top:0px; left:0px; margin: 0px; padding: 0px;";
			} else {
				html_code = html_code + "height:100%; margin: 0px; padding: 0px;";
			}
		} else {
			html_code = html_code + "height:100%; margin: 0px; padding: 0px;";
		}
		if (!(typeof this.prop.style==="undefined")) {
			html_code = html_code +this.prop.style;
		} 
		html_code = html_code +"' ";
		
	
		html_code=html_code+" >";
		
	} else {	
		html_code="<div id='elem"+this.id+"' class='gridlayout' >"; // identify class layout to handle resize
	}
	
	
	for (var i = 0; i < this.width; i++) {
	
		var aBlockWidth = 0;
	
		if (!(typeof this.columnsProperties[i] === "undefined")) { 
	
			if ((this.columnsProperties[i].vertical=="top") || (this.columnsProperties[i].vertical=="bottom")) {
				html_code = html_code + "<div class='gLB' style='width:"+aBlockWidth+"px; top: 0px; height: 100%'>";
			} else if (this.columnsProperties[i].vertical=="center") {
				html_code = html_code + "<div class='gLB' style='width:"+aBlockWidth+"px; top: 0px; height: 100%' >";
			} else {
				html_code = html_code + "<div class='gLB' style='width:"+aBlockWidth+"px; top: 0px; height: 100%; ' >";
			}
		
		} else {
			html_code = html_code + "<div class='gLB' style='width:"+aBlockWidth+"px; top: 0px; height: 100%; ' >";
		}
		
		for (var j = 0; j < this.height; j++) {
		
			if (typeof this.container[i][j] === "undefined"){
				// Fill nothing
			} else {
				//console.log("TypeOf Elem:"+(this.container[i][j].element instanceof ContainerElement));
				
				this.container[i][j].element = this.checkWrapping(this.container[i][j].element);
										
				html_code = html_code + "<div class='gLC' style='left:0px; top:0px;"+
										(!(typeof this.container[i][j].style==="undefined")?this.container[i][j].style:"")+"'>"+this.container[i][j].element.implement()+"</div>";
			}
		}
		
		html_code = html_code + "</div>";
		
		/*
		if (!(typeof this.columnsProperties[i] === "undefined")) { 
		
			if ((this.columnsProperties[i].vertical=="top") || (this.columnsProperties[i].vertical=="bottom")) {
				html_code = html_code + "</div>";
			} else if (this.columnsProperties[i].vertical=="center") {
				html_code = html_code + "</div>";
			} else {
				html_code = html_code + "</div>";
			}
		
		} else {
		
			html_code = html_code + "</div>";
		
		}
		*/
		
	}
	html_code = html_code + "</div>";
	
	return html_code;
	
}


GridLayout.prototype.afterInsert = function(){

	//console.log("GridLayout after insert for element :"+this.element);
	// Add prop to the element
	var aElement = $(this.element);
	aElement.data('prop', this.prop);
	aElement.data('numberOfColumns', this.width);
	aElement.data('columnProperties', this.columnsProperties);
	
	for (var i = 0; i < this.width; i++) {
	
		for (var j = 0; j < this.height; j++) {
		
			if (!(typeof this.container[i][j] === "undefined")){
			
				var id;
				if (this.container[i][j].element instanceof MapperElement) {
					id = "#"+this.container[i][j].element.id; 
				} else {
					id = "#elem"+this.container[i][j].element.id; // Setup y position to the cell element
				}
				$(id).closest(".gLC").data('y',j); // Setup cell position
				
				if (valid(this.prop)) {
				
					if (valid(this.prop.wrap)) {
						$(this.container[i][j].element.element).data('parentWrap',this.prop.wrap);
					}
				
				}
				
				this.container[i][j].element.afterInsert();
			}
			
		}
		
	}

	// TODO inversion on resize handling when wrapping mode
	//handleResize(); // adding a layout can modify all layout sizes
					  // i.e adding a scrollbar
	
	GridLayout.resize(this.element);
	
}

GridLayout.truncate = function(n) {
  return Math[n > 0 ? "floor" : "ceil"](n);
}

GridLayout.checkNeededResize = function(iLayout,iForceResize) {

	var aLayout = $(iLayout);
	var elem = aLayout.data("parentW");
	var pParentWidth = (valid(elem)?elem:0);
	elem = aLayout.data("parentH");
	var pParentHeight = (valid(elem)?elem:0);
	
	var parentWidth = aLayout.parent().width();
	var parentHeight = aLayout.parent().height();

	//console.log("parentW:"+parentWidth+" pPW:"+pParentWidth);
	//console.log("parentH:"+parentHeight+" pPW:"+pParentHeight);

	var result = true;
	if ((parentWidth == pParentWidth) && (parentHeight==pParentHeight)) {
		result = false;
	} 

	//console.log("Result:"+result);

	aLayout.data("parentW",parentWidth);
	aLayout.data("parentH",parentHeight);

	return result || iForceResize ;

}

GridLayout.resize = function(iLayout,iChain,iForceResize){

	// iLayout is an id
	var aLayout = $(iLayout);

	var prop = aLayout.data('prop');
	var width = aLayout.data('numberOfColumns');
	var columnProperties = aLayout.data('columnProperties');
	
	var parentWrap = (valid(aLayout.data('parentWrap'))?aLayout.data('parentWrap')==true:false);
	
	var blocks = aLayout.children(".gLB");
	
	// defined booleans
	var propDefined = (!(typeof prop==="undefined")) && (prop!=null);
	var fixedDefined = (propDefined?!(typeof prop.fixed==="undefined"):false);
	var percentDefined = (propDefined?!(typeof prop.percent==="undefined"):false);
	var wrapDefined = (propDefined?!(typeof prop.wrap==="undefined"):false);
	
	var aElementWidth = aLayout.parent().width();
	var maxHeight = 0;
	
	
	var wrap = false;
	var fullscreen = false;
	if (propDefined) {
	
		// Wrap elements to fixed sizes
		if (wrapDefined){
		
			if (prop.wrap==true) {
				wrap = true;
			}
			
			
			if (wrap) {
			
				prop.fixed = [];
				fixedDefined = true;
			
				for (var j=0; j<blocks.length; j++) {
				
					var maxWidth = 0;
					$.each($(blocks[j]).children(".gLC"), function(key,value) {
						handleDependentResize($(value),iChain,iForceResize);
						if ($(value).children().outerWidth()>maxWidth) {
							maxWidth = $(value).children().outerWidth();
						}
					});
					prop.fixed[j] = maxWidth;
					
				
				}

			} 
						
		} 
	
		if (valid(prop.fullscreen)){
		
			if (prop.fullscreen) { // this will probably request a new size calculation
				
				//console.log("Fullscreen mode");
				fullscreen = true;
				
				var aElementWidth = $(window).width()-2;
				var aElementHeight = $(window).height()-2; // TODO replace by border size
				var aCWidth= aLayout.width();
				var aCHeight = aLayout.height();
				
				// Check min size of fixed size for width
				if (fixedDefined) {
					//var sumFixed = prop.fixed.reduce(function(a, b) { return a + b; }, 0); // to add at the end of IE8 :-)
					var sumFixed = 0;
					for (var i=0; i<prop.fixed.length;i++){
						sumFixed+=prop.fixed[i];
					}
					if (sumFixed>aElementWidth) {
						aElementWidth = sumFixed;
					}
				}
				
				aMaxHeight = aElementHeight;
				
			}
		
		
		}
		
		if ((!wrap) && (!fullscreen)) {
		
			if (!GridLayout.checkNeededResize(aLayout,iForceResize)) {
				return;
			}
		
		}	
		
	}
	
	// Comput the max height of each blocks
	var columnLead = 0;
	var nPercent = 0;
	var nWrap = 0;
	var nDefined = 0;
	//console.log("Compute block height");
	for (var j=0; j<blocks.length; j++) {
	
		var cprop;
		if (valid(columnProperties)) { cprop = columnProperties[j];}
		var cpropdefined = false;
		var fixed = false;
		var percent = false;
		var hWrapDefined = false;
		if (!(typeof cprop==="undefined")) {
			cpropdefined = true;
			fixed = valid( cprop.fixed );
			percent	= valid( cprop.percent );
			hWrapDefined = valid( cprop.wrap );
		}
	
		if (cpropdefined){
		    nDefined++;
			if (percent) {
			   nPercent++;
			}
			if (hWrapDefined) {
			   nWrap++;
			}
		}
	
		// Wrap Height
		if (hWrapDefined){
		
			if (cprop.wrap==true) {
		
				//console.log("Column ("+j+") start wrapping ");
		
				cprop.fixed = [];
			
				$.each($(blocks[j]).children(".gLC"), function(key,value) {
					var y = $(value).data('y'); // get height position of the cell
					cprop.fixed[y] = $(value).children().outerHeight(); // force cell height
				});

			}
		
		}
	
		var size = 0; // Define height of each cell
		$.each($(blocks[j]).children(".gLC"), function(key,value) {
			
			var aValue = $(value);
			var y = aValue.data('y');
			// Column properties
			if (cpropdefined && fixed) {
				if (valid(cprop.fixed[y])) {
					aValue.height(cprop.fixed[y]);
					size += cprop.fixed[y];
				}
			} 
			
			
		});
		
		if (size>maxHeight) {
			maxHeight = size;
			columnLead = j;
		}
		
	}
	
	//console.log("After fixed + wrap Max Height:"+maxHeight);
	
	var aWantedDefine = $(iLayout).parent().height();
	if (propDefined) {
		if (valid(prop.fullscreen)){
			if (prop.fullscreen) {
				aWantedDefine = $(window).height();
			}
		} 
	}
	
	// In Wrap mode we have to keep max height
	if ((nWrap!=nDefined) && (!parentWrap)) {
		maxHeight=aWantedDefine;
	}	
	
	//console.log("After defining fixed height + wrap wanted Height:"+maxHeight);
	
	// Special case for fixed height definition
	for (var j=0; j<blocks.length; j++) {
	
		var cprop;
		if (valid(columnProperties)) { cprop = columnProperties[j];}
		var cpropdefined = false;
		var fixed = false;
		if (valid(cprop)) {
			cpropdefined = true;
			fixed = valid(cprop.fixed);
		}
	
		if (cpropdefined && fixed) { // If column fixed
	
			var numberNotDefined = 0;
			var cumulHeight = 0;
			$.each($(blocks[j]).children(".gLC"), function(key,value) {
			
				var aValue = $(value);
				var y = aValue.data('y');
				// Column properties			
				if (valid(cprop.fixed[y])) {
					if (cprop.fixed[y]==0) {
						numberNotDefined++;
					} else {
						cumulHeight+=aValue.height();
					}
				}
				 
			});
		
			var aPosY = 0;
			var aRemainder = (maxHeight-cumulHeight);
			if (aRemainder<0) {
				aRemainder = 0;
			}
			if (cpropdefined) {
				if ((numberNotDefined==0) && (cprop.vertical=='center')){
					aPosY = aRemainder/2;
				}
			}
	
			$.each($(blocks[j]).children(".gLC"), function(key,value) {
				var aValue =  $(value);
				var y = aValue.data('y');
				
				// Column properties
				if (valid(cprop.fixed[y])) {
					
					if (cprop.fixed[y]==0) {
						aValue.height(aRemainder/numberNotDefined);
					}
						
					$(value).css({ top: aPosY });
					aPosY = aPosY + aValue.height(); // Set top position
				
				} 
			});
		
		}
		
	}
	
	// Try to handle undefined size for column
	// By default we want to fill 100% of the parent
	// so undefined size for column will complete the
	// parent size.
	var numberUndefined = 0;
	var sizeDefined = 0;
	// TODO record precalculated
	if (fixedDefined || percentDefined) {
		for (var i=0; i<blocks.length; i++) {
			
				if (percentDefined) {
				
					if (!(typeof prop.percent[i]==="undefined")) { // Percentage definition
						if (prop.percent[i]!=0) {
							sizeDefined += GridLayout.truncate((prop.percent[i]*aElementWidth)/100);
						} else {
							numberUndefined++;
						}
					} else {
						numberUndefined++;
					}
				
				} else if (fixedDefined) {
				
					if (!(typeof prop.fixed[i]==="undefined")) { // Fixed definition 
						if (prop.fixed[i]!=0) {
							sizeDefined += prop.fixed[i];
						} else {
							numberUndefined++;
						}
					} else {
						numberUndefined++
					}
				
				} 
				
		}
	}
	
	var remainder = ((aElementWidth-sizeDefined)/numberUndefined);
	if (remainder<0) { remainder = 0; };
	
	// Resize blocks 
	var relativePosition = 0;
	var maxWidth = 0;
	var realMaxHeight = 0;
	for (var i=0; i<blocks.length; i++) {
	
		var aBlockWidth = 0;
		if (percentDefined) {
		
			if (!(typeof prop.percent[i]==="undefined")) { // Percentage definition
				if ( prop.percent[i] != 0 ) {
					aBlockWidth = GridLayout.truncate((prop.percent[i]*aElementWidth)/100);
				} else {
					aBlockWidth = remainder;
				}
			} else {
				aBlockWidth = remainder;
			}
		
		} else if (fixedDefined) {
		
			if (!(typeof prop.fixed[i]==="undefined")) { // Fixed definition 
				if (prop.fixed[i] !=0 ) { 
					aBlockWidth = prop.fixed[i];
				} else {
					aBlockWidth = remainder;
				}
			} else {
				aBlockWidth = remainder;
			}
		
		} else { // If nothing divide the size in equal parts
			aBlockWidth = aElementWidth/width;
		}
		
		// Block width compute.
		var aCBlockWidth = $(blocks[i]).width();
		$(blocks[i]).width(aBlockWidth);
		$(blocks[i]).css(/*{ "top": $(blocks[i]).position().top+"px",*/ { left: relativePosition+"px"});
		relativePosition = relativePosition+aBlockWidth;
		maxWidth = maxWidth + aBlockWidth; // = relative position
		
		// ############################## COMPUT PERCENT HEIGHT ###################################
		// Comput Height of elements for percent definition
		var cprop;
		if (valid(columnProperties)) { cprop = columnProperties[i];}
		var cpropdefined = false;
		var fixed = false;
		var percent = false;
		if (valid(cprop)) {
			cpropdefined = true;
			fixed = valid(cprop.fixed);
			percent	= valid(cprop.percent);
		}
	
		// Pre build values;
		var heightAcc = 0;
		var miss = 0;
		var cells = $(blocks[i]).children(".gLC");
		$.each(cells, function(key,value) {
		
			var aValue = $(value);
			var y = aValue.data('y');
			
			// Column properties
			if (cpropdefined && percent) {
				if (valid(cprop.percent[y])) {
					var val = GridLayout.truncate((cprop.percent[y]*maxHeight)/100);
					//console.log("Defined val "+val);
					if (cprop.percent[y]!=0) {
						aValue.height(val);
						aValue.width(aBlockWidth);
						heightAcc+=val;
					} else {
						miss++;
					}
					
				} 
			} else {
			
				heightAcc+=aValue.height();
			
			}
	
		});
		var heightRemainder = (maxHeight-heightAcc);
		if (miss!=0) { heightRemainder = heightRemainder/miss; }
		if (heightRemainder<0) {heightRemainder=0;}
	
		// This is needed to center cells in the layout (top, center)
		// For percent column definition, define center only if no cells 
		// defined with 0 height i.e. no automatic resize
		// If miss == 0 we have to center if needed
		
		var aPosY = 0;
		if (cpropdefined) {
			if ((miss==0) && (cprop.vertical=='center')){
				aPosY = heightRemainder/2;
			}
		}
	
		$.each(cells, function(key,value) {	
			var y = $(value).data('y');
			// Column properties
			if (cpropdefined && percent) {
				if (!(typeof cprop.percent[y]==="undefined")) { 
				
					if (cprop.percent[y]==0) { 
						$(value).height(heightRemainder);
						$(value).width(aBlockWidth);
					} 

					//console.log("PosY:"+aPosY);
					$(value).css({ top: aPosY });
					aPosY = aPosY + $(value).height(); // Set top position
					$(value).width(aBlockWidth); // Size block with
					
				}
				
			} else if (cpropdefined && fixed) {
			
				$(value).width(aBlockWidth); 
				aPosY = aPosY + $(value).height();
			
			} else {
			
				$(value).css({ top: aPosY }); // To Center
				aPosY = aPosY + $(value).height(); // Set top position
				$(value).width(aBlockWidth); 
			
			}
			
		});
		// aPosY contains the real height // cumulative height
		
		// end of height definition
		// ############################## END COMPUT PERCENT HEIGHT ################################
		
		if (propDefined) {
			if (!prop.fullscreen) {	
				//
				$(blocks[i]).height(maxHeight);
				if (maxHeight>realMaxHeight) {
					realMaxHeight = maxHeight;
				}

			} else { // If fullscreen, if size of blocks > document.height()
					 // i.e fixed case or may be wrapped mode ?
					 // this permit scroll bar when override != hidden	
			
				if (aPosY!=0) {
					$(blocks[i]).height(aPosY);
					if (aPosY>realMaxHeight) {
						realMaxHeight = aPosY;
					}
				} else {
					$(blocks[i]).height(maxHeight);
					if (maxHeight>realMaxHeight) {
						realMaxHeight = maxHeight;
					}
				}
				
			}
		} else {
			$(blocks[i]).height(maxHeight);
			if (maxHeight>realMaxHeight) {
				realMaxHeight = maxHeight;
			}
		}

		//console.log("remainder:"+heightRemainder);
		//console.log("defined aPosY:"+aPosY);	
		//console.log("defined MaxHeight:"+maxHeight);
		//console.log("realMaxHeight:"+realMaxHeight);
	
	}
	
	if (propDefined) {
		if (wrapDefined) {
			aLayout.width(maxWidth);
			aLayout.height(maxHeight);
		}
		if (parentWrap) {
			aLayout.width(maxWidth);
			aLayout.height(maxHeight);
		}
		if (prop.fullscreen) {
			aLayout.height(maxHeight);
			aLayout.width(maxWidth);
			if (realMaxHeight>maxHeight) {
				aLayout.css("overflow-y","scroll");
				aLayout.css("overflow-x","hidden");
			} else {
				aLayout.css("overflow-y","hidden");
				aLayout.css("overflow-x","hidden");
			}
		}
	}
	
	if (!wrapDefined) {
		handleDependentResize(aLayout,iChain,iForceResize);
	} else {
		if (prop.wrap==false) {
			handleDependentResize(aLayout,iChain,iForceResize);
		}
	}
	
	Element.resizeSpecialElements(iLayout,iChain,iForceResize);
}

GridLayout.prototype.add = function (iX,iY,iElement) {
	
	var defined = !(typeof this.container[iX][iY]==="undefined");
	if (defined) { 
		this.container[iX][iY] = {
			id : elementID,
			classtype : this.container[iX][iY].classtype,
			style : this.container[iX][iY].style,	
			element : iElement
		};
	} else {
		this.container[iX][iY] = {
			id : elementID,
			element : iElement
		};
	}
	elementID++;
	return this;
	
}

GridLayout.prototype.getInnerElements = function () {

	var elements = [];

	for (var i = 0; i < this.width; i++) {
	
		for (var j = 0; j < this.height; j++) {
		
			if (!(typeof this.container[i][j] === "undefined")){
			
				elements.push(this.container[i][j].element);
			
			}
			
		}
		
	}

	return elements;
}

GridLayout.prototype.addColumnProperties = function (iX, iProperties){
	
	this.columnsProperties[iX] = iProperties;
	return this;
	
}

GridLayout.prototype.addProperties = function (iX,iY,iProperties) {

	var defined = !(typeof this.container[iX][iY]==="undefined");
	if (defined) {
		this.container[iX][iY] = {
			id : elementID,
			classtype : iProperties.classtype,
			style : iProperties.style,	
			element : this.container[iX][iY].element
		};
	} else {
		this.container[iX][iY] = {
			id : elementID,
			classtype : iProperties.classtype,
			style : iProperties.style
		};
	}
	elementID++;
	return this;
}
// ######################## DEFINE LINEARLISTLAYOUT ##########################

var LinearListLayout = function(iOrientation,iMode){
	Layout.call(this);
	this.orientation = iOrientation;
	this.mode = iMode;
	this.innerlayout = null;
	this.elementList = [];
	this.elementSize = [];
}
LinearListLayout.prototype = new Layout();
LinearListLayout.prototype.constructor = LinearListLayout;

LinearListLayout.prototype.implement = function () {

	var html_code = "<div id='elem"+elementID+"' class='linearlistlayout'>";
	this.id = elementID;
	this.element = "#elem"+this.id;
	elementID++;
	
	if (this.elementList.length>0) {
	
		if (this.orientation=="horizontal") {
			
			var halignDef = (!(typeof this.halign==="undefined")?this.halign:"left");
			var gridSize = (halignDef=="center"?this.elementList.length+2:(halignDef=="right"?this.elementList.length+1:this.elementList.length));
			
			var shift = 0;
			var aElementSize = [];
			if ((halignDef=="center") || (halignDef=="right"))  {
				shift = 1;
				aElementSize.push(0);
			}
			aElementSize = aElementSize.concat(this.elementSize);
			if (halignDef=="center")  {
				aElementSize.push(0);
			}
			//console.log("Given element size:"+this.elementSize);
			//console.log("Element Size Array:"+aElementSize);
			//console.log("Grid Size:"+gridSize);
			//console.log("Mode:"+this.mode);
			
			this.innerlayout = new GridLayout(gridSize,1);
			if (this.mode=="wrap") { // Force also column properties to wrap
				this.innerlayout.setProperties({ // TODO define fullscreen value
					style:' width:100%;',
					wrap: true
				});
			} else {
				this.innerlayout.setProperties({ // TODO define fullscreen value
					style:' width:100%;',
					fixed: aElementSize
				});
				//console.log("ElementSizes:");
				//console.log(aElementSize);
			}
			
			var valignDef = (!(typeof this.valign==="undefined")?this.valign:"top");
			
			for (var i=0; i<this.elementList.length; i++){
				//console.log("Add column "+(i+shift));
				this.innerlayout.add(i+shift,0,this.elementList[i]);
				if (this.mode=="wrap") { // this check wrapping mode
					this.innerlayout.addColumnProperties(i+shift,{vertical: valignDef,wrap:true}); 
				} else {
					this.innerlayout.addColumnProperties(i+shift,{vertical: valignDef,percent:[100]}); 
				}
			}
			
			//console.log(this.innerlayout);
			
			html_code += this.innerlayout.implement();
			
		} else if (this.orientation=="vertical") {
		
			this.innerlayout = new GridLayout(1,this.elementList.length);
			if (this.mode=="wrap") { // Force also column properties to wrap
				this.innerlayout.setProperties({ // TODO define fullscreen value
					style:' width:100%;',
					wrap: true
				});
			} else {
				this.innerlayout.setProperties({ // TODO define fullscreen value
					style:' width:100%;',
					percent: [100]
				});
			}
			
			//var alignDef = (!(typeof this.align==="undefined")?this.align:"top");
			
			for (var i=0; i<this.elementList.length; i++){
				//console.log("Add line "+i);
				this.innerlayout.add(0,i,this.elementList[i]);
				if (this.mode=="wrap") { // this check wrapping mode
					this.innerlayout.addColumnProperties(i,{vertical: "top",wrap:true}); 
				} else {
					this.innerlayout.addColumnProperties(i,{vertical: "top",fixed:this.elementSize}); 
				} 
			}
			
			//console.log(this.innerlayout);
			
			html_code += this.innerlayout.implement();
		
		}
		
	} else {
		this.innerlayout = new GridLayout(0,0);
	}
	
	html_code += "</div>";
	
	return html_code;
	
}

LinearListLayout.prototype.refresh = function() {

	var implemented=(!(typeof this.implemented==="undefined")?true:false);
	if (implemented) {
		//console.log("Insert for LinearLayout:"+this.element);
		//console.log($(this.element));
		var aElem = $(this.element).parent();
		aElem.empty();
		var html_code = this.implement();
		//console.log(html_code);
		aElem.append(html_code);
		
		forceResize = true;
		this.afterInsert();
		forceResize = false;
		
	}

}

LinearListLayout.prototype.addElement = function(iElement,iSize){

	this.elementList.push(iElement);
	this.elementSize.push(iSize);
	
}

LinearListLayout.prototype.removeElementByTag = function(iTag) {

	var pos = -1;
	for (var i=0; i<this.elementList.length; i++) {
		var aElement = this.elementList[i];
		if (valid(aElement.tag)) {
			if (aElement.tag==iTag) {
				pos=i;
				break;
			}
		}
	}

	if (pos!=-1) {
		this.elementList.splice(pos,1);
		this.elementSize.splice(pos,1); 
		this.refresh();
	}

}

LinearListLayout.prototype.getInnerElements = function () {
	return this.elementList;
}

LinearListLayout.prototype.pushElement = function(iElement,iSize){

	this.addElement(iElement,iSize);
	this.refresh();
	
}

LinearListLayout.prototype.dropAll = function(){
	
	this.elementList = [];
	this.elementSize = [];
	
}

LinearListLayout.prototype.clear = function(){
	
	this.dropAll();
	this.refresh();
	
}

LinearListLayout.prototype.setAlignment = function (iHAlign,iVAlign){
	this.valign=iVAlign;
	this.halign=iHAlign;
	return this;
}

LinearListLayout.prototype.afterInsert = function (){
	//console.log("LinearListLayout after insert");
	//console.log(this.element);
	//this.innerlayout.element = this.element;
	
	this.innerlayout.afterInsert();
	
	$(this.element).data("mode",this.mode);
	$(this.element).data("halign",this.halign);
	$(this.element).data("valign",this.valign);
	
	LinearListLayout.resize($(this.element),"",0);
	
	this.implemented = true;
	
}

LinearListLayout.resize = function(iElement,iChain,iForceResize){
	
	//console.log("LinearListLayout resize");
	//console.log($(iElement));
	//console.log("width:"+$(iElement).width());
	//console.log("height:"+$(iElement).height());
	
	var mode = $(iElement).data("mode");
	
	var aElement = $(iElement);
	if (mode=='wrap') {
		//console.log("LinearListLayout wrap");
		
		aElement.width( aElement.parent().width() );
		aElement.height( aElement.parent().height() );
		
		handleDependentResize($(iElement),iChain,iForceResize);
		
		aElement.width( aElement.find(".gridlayout").outerWidth() );
		aElement.height( aElement.find(".gridlayout").outerHeight() );
	
	} else {
		aElement.width( aElement.parent().width() );
		aElement.height( aElement.parent().height() );
		handleDependentResize($(iElement),iChain,iForceResize);
	}
	
	Element.resizeSpecialElements(iElement,iChain,iForceResize);
}
// ######################## DEFINE MENUBAR ELEMENT ##########################

var MenuBarElement = function(){
	Element.call(this);
	this.inner = new LinearListLayout("horizontal","wrap");
	this.inner.setAlignment("left","center");
	this.elementList = {};
}
MenuBarElement.prototype = new Element();
MenuBarElement.prototype.constructor = MenuBarElement;

MenuBarElement.prototype.setAlign = function(iAlign){
	this.inner.setAlignment(iAlign,"center");
	return this;
}

MenuBarElement.prototype.setTextColor = function(iColor){
	this.textColor = iColor;
	return this;
}

MenuBarElement.prototype.setFontSize = function(iSize){
	this.fontSize = iSize;
	return this;
}

MenuBarElement.prototype.setMouseOverColor = function(iColor){
	this.mouseOverColor = iColor;
	return this;
}

MenuBarElement.prototype.setMouseOverBkColor = function(iColor){
	this.mouseOverBkColor = iColor;
	return this;
}

MenuBarElement.prototype.setPadding = function(iPadding){
	this.padding = iPadding;
	return this;
}

MenuBarElement.prototype.setVerticalBarStyle = function(iStyle){
	this.vStyle = iStyle;
	return this;
}

MenuBarElement.prototype.setVerticalBarColor = function(iColor){
	this.vColor = iColor;
	return this;
}

// Percent
MenuBarElement.prototype.setVerticalBarHeight = function(iHeight){
	this.vHeight = iHeight;
	return this;
}

MenuBarElement.prototype.setVerticalBarWidth = function(iWidth){
	this.vWidth = iWidth; 
	return this;
}

MenuBarElement.prototype.implement = function () {

	var html_code = this.inner.implement();
	
	// hack to handle correctly sizing
	this.id = this.inner.id;
	this.element = this.inner.element;
	
	//console.log("MenuBarElement");
	//console.log(this.id);
	//console.log(this.element);
	
	return html_code;
	
}

MenuBarElement.prototype.pushElements = function(iElements,iTranslates){


	for (var i=0; i<iElements.length; i++) {
	
		var translate;
		if (valid(iTranslates)) {
			translate = iTranslates[i];
		}
		if (i==0) {
			this.pushElement(iElements[i],translate);
		} else {
			this.setVerticalBar();
			this.pushElement(iElements[i],translate);
		}
	
	}

}

MenuBarElement.prototype.pushElement = function(iText,iTranslate){

	var aTextElement = new TextElement(iText);
	
	if (valid(this.textColor)) {
		aTextElement.setColor(this.textColor);
	}
	if (valid(this.fontSize)) {
		aTextElement.setFontSize(this.fontSize);
	}
	if (valid(this.mouseOverColor)) {
		aTextElement.setMouseOverColor(this.mouseOverColor);
	}
	if (valid(this.mouseOverBkColor)) {
		aTextElement.setMouseOverBkColor(this.mouseOverBkColor);
	}
	if (valid(this.padding)) {
		aTextElement.setStyle("padding:"+this.padding+";");
	}
	if (valid(iTranslate)) {
		aTextElement.translate(iTranslate);
	}
	
	this.inner.pushElement(aTextElement);
	
	this.elementList[iText] = aTextElement; // This is to keep reference on menu bar elements
	
}

MenuBarElement.prototype.setVerticalBar = function(){

	var aVerticalBar = new VerticalBarElement();
	
	if (valid(this.vStyle)) {
		aVerticalBar.setStyle(this.vStyle);
	}
	
	if (valid(this.vColor)) {
		aVerticalBar.setColor(this.vColor);
	}
	
	if (valid(this.vWidth)) {
		aVerticalBar.setWidth(this.vWidth);
	}
	
	if (valid(this.vHeight)) {
		aVerticalBar.setHeight(this.vHeight);
	}
	
	this.inner.pushElement(aVerticalBar);

}

MenuBarElement.prototype.getInnerElements = function () {
	return this.inner.getInnerElements();
}

MenuBarElement.prototype.afterInsert = function (){
	
	this.inner.afterInsert();
	LinearListLayout.resize($(this.inner.element));
	
}
/* Layout Parser JBV 2012-2013 */

var LayoutParser = function(){
	
	this.references = {};
	this.elements = [];
	this.taggedelements = {};
	this.elementTokens = [ "define", "include", "import", // Structure management
						   "if", "loop",  // Data tree path exploration 
						   "gridlayout", "text", "textinput", "image" , "container", "empty", "linearlistlayout", "scrollcontainer", "mapper", "verticalbar", "button", "slider", // Elements management
						   "menubar" // Extended Elements
						 ];
	this.init = false;
	this.elementParser = {};
	this.elementPost = {};
	
} 

LayoutParser.prototype.recordFunctions = function() {
	
	// Structure management
	this.elementParser[ "define" ] = this.parseDefine;
	this.elementParser[ "include" ] = this.parseInclude;
	this.elementParser[ "import" ] = this.parseImport;
	
	// If/Loop
	this.elementParser[ "loop" ] = this.parseLoop;
	
	// Elements management
	this.elementParser[ "gridlayout" ] = this.parseGridLayout;
	this.elementParser[ "text" ] = this.parseTextElement;
	this.elementParser[ "textinput" ] = this.parseTextInputElement;
	this.elementParser[ "image" ] = this.parseImageElement;
	this.elementParser[ "container" ] = this.parseContainerElement;
	this.elementParser[ "empty" ] = this.parseEmptyElement;
	this.elementParser[ "linearlistlayout" ] = this.parseLinearListLayout;
	this.elementParser[ "mapper" ] = this.parseMapperElement;
	this.elementParser[ "scrollcontainer" ] = this.parseScrollContainerElement;
	this.elementParser[ "verticalbar" ] = this.parseVerticalBarElement;
	this.elementParser[ "menubar" ] = this.parseMenuBarElement;
	this.elementParser[ "slider" ] = this.parseSliderElement;
	
	this.elementPost[ "textinput" ] = this.postTextInputElement;
	
}

LayoutParser.prototype.load = function(iLayoutURL,iDataTree) {

	var aData;
	$.ajaxSetup( { "async": false } );
	$.getJSON(iLayoutURL, function(data) {
		aData = data;
	});
	$.ajaxSetup( { "async": true } );

	if (valid(aData)) {
		return this.parseLayout(aData,iDataTree);
	}
	return;
}

LayoutParser.prototype.parseLayout = function(iData, iDataTree) {

	if (!this.init) {
		this.recordFunctions();
		this.init = true;
	}

	this.references = {};
	this.elements = [];
	this.taggedelements = {};
	
	var aLayout = this.parse(iData, iDataTree);
	this.postProcess(); // build links between objects
	
	aLayout.tag = this.taggedelements;
	//console.log(aLayout.tag);
	
	return aLayout;
	
}

// This function go through the data tree i.e I will receive "node1.node2"
// So the position in the data tree will be 
LayoutParser.prototype.setPosition =  function(iSteps,iDataTree){

	//console.log("Steps:"+iSteps);
	//console.log(iSteps);
	var aSteps = iSteps.split(".");
	var aDataTree = iDataTree;
	//console.log("setPosition");
	for (var i=0; i<aSteps.length; i++) {
		//console.log(aDataTree);
		//console.log(aSteps[i]);
		
		var aStep = aSteps[i];
		var startBracket = aStep.search("\\[");
		var stopBracket = aStep.search("\\]");
		var haveIndex = false;
		var index;
		if ((startBracket!=-1) && (stopBracket!=-1)) {
			haveIndex = true;
			var extract = aStep.substring(startBracket+1,stopBracket);
			console.log("Extract:"+extract);
			index = parseInt( extract, 10);
			
			aStep =  aStep.substring(0,startBracket);
			console.log("New aStep:"+aStep);
		}		
		
		if (valid(aDataTree[aStep])) {
			
			aDataTree = aDataTree[aStep];
			if (haveIndex) {
			   aDataTree = aDataTree[index];
			} 
			
		} else {
			logError("LayoutParser : Error when parsing layout, can't reach the data "+iSteps+".", iDataTree);
			
			return;
		}
	}
	return aDataTree;
	
}

// Concat several data
LayoutParser.prototype.concat = function(iElement,iDataTree) {
	
	var aElements = [];
	for (var i=0; i<iElement.length; i++) { // TODO test if iElement is an array
		console.log(iElement[i]);
		aElements.push(this.convertData(iElement[i],iDataTree));
	}
	return aElements.join("");
	
}

// This will shorten layout implementation
// Here strict implementation to get data from the tree use "." 
// Directly after json separator i.e  ".data" will search in data tree 
// ".node.data" will work also
// " .data" will be considered as direct data
// "\.data" will be considered as direct data ".data"
LayoutParser.prototype.convertData = function(iElement,iDataTree) {
	//console.log("convertData :"+iElement);
	if (valid(iElement.agreg)) { // ask to concat all data
	
		//console.log("agreg");
		return this.concat(iElement.agreg, iDataTree);
	
	} else if (valid(iElement.eval)) { // ask to eval string
	
		//console.log("evaluate");
		return eval(this.convertData(iElement.eval,iDataTree));
	
	} else if (valid(iElement.formatdate)) {
		
		console.log("Formatting a date :");
		
		if (valid(iElement.formatdate.from) && valid(iElement.formatdate.to) && valid(iElement.formatdate.data)) {
		
			var aData = this.convertData(iElement.formatdate.data,iDataTree);
			var aDate = moment(aData,iElement.formatdate.from);
			console.log(moment().format());
			return aDate.format(iElement.formatdate.to);
		
		} 
		
	} else { // Consider iElement has a string

		/*console.log("Element");
		console.log(iElement);
		console.log(typeof iElement);*/
	
		if ( typeof iElement === "string") {
			if (iElement.indexOf(".")==0) { // Consider this as data element
				var name = iElement.substring(1);
				return this.setPosition(name,iDataTree);
			} else if (iElement.indexOf("\\.")==0) { // Consider this as direct data
				return iElement.substring(1);
			} else {
				return iElement;
			}
		} else {
			return iElement;
		}
	
	}
}

LayoutParser.prototype.parse = function(iData,iDataTree) { // TODO Create a hashmap of independent parsers

	for (var i=0; i<this.elementTokens.length; i++) {
	
		var aElement = iData[this.elementTokens[i]];
		this.decodingFunction = this.elementParser[this.elementTokens[i]];
	
		if (valid(aElement) && valid(this.decodingFunction)) {
		
			var aDataTree = iDataTree;
			if (valid(aElement["use"])) { // Detect "use" term in the element, this will go through the data tree
				aDataTree = this.setPosition( aElement["use"] , iDataTree );
			}
		
			var aParsedElement = this.decodingFunction(aElement,aDataTree);
		
			if (valid(aElement.tag)) {
				//console.log("record Tag:"+aElement.tag);
				var aTag = this.convertData(aElement.tag,aDataTree);
				this.taggedelements[aTag] = {
					content : aElement,
					element : aParsedElement
				}
				aParsedElement.tag = aTag; 	
			}
			
			if (valid(aElement.attach)) {
				//console.log("record Tag:"+aElement.tag);
				aParsedElement.attached = this.convertData(aElement.attach,aDataTree);	
			}
		
			this.elements.push({ content :  aElement, element : aParsedElement });
		
			return aParsedElement;
		
		} 
	
	}
	
	logError("LayoutParser : Error when parsing layout, element not found .", iData);
	return;

}



// Define for defining references + main layout
// can be used for duplicated elements
LayoutParser.prototype.parseDefine = function(iData,iDataTree) {
	
	if (valid(iData.references)) {
		
		var aReferences = iData.references;
		for (var i=0; i<aReferences.length; i++) {
		
			var reference = aReferences[i];
			
			if (valid(reference.name) && valid(reference.content)) {
				
				this.references[reference.name] = reference.content;
				
			} else {
			
				logError("LayoutParser : Error no reference name or content defined  :",aReference);
				return;

			}
		
		}	
	
	} // no reference defined
	
	// at the end we have to parse the main layout
	if (valid(iData.layout)) {
	
		return this.parse(iData.layout,iDataTree);
	
	} else {
		logError("LayoutParser : Error no define layout defined  :",iData);
		return;
	}
	
}

// Include element ... will include a reference at that point
LayoutParser.prototype.parseInclude = function(iData,iDataTree) {

	if (valid(iData.name)) {
	
		if (valid(this.references[iData.name])) {
			return this.parse(this.references[iData.name],iDataTree);
		} else {
			logError("LayoutParser : Error no reference defined for name "+iData.name+" layout defined  :",iData);
			return;
		}
		
	} else {
		logError("LayoutParser : Error in include no name defined  :",iData);
		return;
	}
	
}

LayoutParser.prototype.parseImport = function(iData,iDataTree) {

	if (valid(iData.url)) {

		var url = iData.url;
	
		var aData;
		$.ajaxSetup( { "async": false } );
		$.getJSON(url, function(data) {
			aData = data;
		});
		$.ajaxSetup( { "async": true } );
	
		return this.parse(aData,iDataTree);
	
	} else {
		logError("LayoutParser : Error in import no url defined  :",iData);
		return;
	}

}

// Loop/If
LayoutParser.prototype.parseLoop = function(iData,iDataTree) {

	// First parse a linear list layout
	var aLayout = this.parseLinearListLayout(iData,iDataTree);

	if (valid(aLayout)) {
		// Then loop on data tree
		if (valid(iData.elements)) {
	
			var aList = this.convertData(iData.elements,iDataTree);
			aList = [].concat( aList );
			console.log(aList);
		
			if (valid(iData.model)){
			
				for (var i=0; i<aList.length; i++) {
					aLayout.pushElement(this.parse(iData.model,aList[i]));
				}
			
			} else {
				logError("LayoutParser : Error in loop no model defined  :",iData);
				return;
			}
		
		} else {
			logError("LayoutParser : Error in loop no elements defined  :",iData);
			return;
		}
	} else {
		logError("LayoutParser : Error in loop layout not defined  :",iData);
		return;
	}
	
	return aLayout;

}


// GridLayout Parser
LayoutParser.prototype.parseGridLayout = function(iData,iDataTree) {
	
	if (valid(iData.dim)) {
	
		if ( (!valid(iData.dim[0])) || (!valid(iData.dim[1])) ) {
			logError("LayoutParser : Error dim not valid in gridlayout :",aColumn);
			return;
		}
	
		var aGridLayout = new GridLayout(this.convertData(iData.dim[0],iDataTree),this.convertData(iData.dim[1],iDataTree));
			
		if (valid(iData.properties)) {
			aGridLayout.setProperties(this.convertData(iData.properties,iDataTree));
		}
		
		var columns = iData.columns;
		if (valid(columns)){
			
			for (var i=0; i<columns.length; i++) {
				var aColumn = columns[i];
				if (valid(aColumn.x) && valid(aColumn.properties)) {
				  aGridLayout.addColumnProperties(this.convertData(aColumn.x,iDataTree),this.convertData(aColumn.properties,iDataTree));	
				} else {
				  logError("LayoutParser : Error no column x or properties defined in gridlayout :",aColumn);
				  return;
				}
			}
			
		}
		
		var cells = iData.cells;	
		if (valid(cells)){
			
			for (var i=0; i<cells.length; i++) {
			
				var aCell = cells[i];
				if (valid(aCell.pos) && valid(aCell.inner)) {
					
					if (valid(aCell.pos[0]) && valid(aCell.pos[1])) {
				
						var aElement = this.parse(aCell.inner,iDataTree);	
						aGridLayout.add(this.convertData(aCell.pos[0],iDataTree),this.convertData(aCell.pos[1],iDataTree),aElement);
					
					} else {
						logError("LayoutParser : Error no pos value defined for gridlayout :",aCell);
						return;
					}
				
				} else {
				  logError("LayoutParser : Error no pos element or (inner component) defined for gridlayout :",aCell);
				  return;
				}
			}
		
		} else {
			logError("LayoutParser : Error no cell defined for gridlayout :",iData);
			return;
		}
			
		return aGridLayout;
			
	} else {
		logError("LayoutParser : Error no width and height defined for gridlayout :",iData);
		return;
	}
	
}

// TextElement Parser
LayoutParser.prototype.parseTextElement = function(iData,iDataTree) {

	if (valid(iData.text)) {
	
		var aTextElement = new TextElement(this.convertData(iData.text,iDataTree));
	
		// fontsize
		if (valid(iData.fontsize)) {
			aTextElement.setFontSize(this.convertData(iData.fontsize,iDataTree));
		}
	
		// position
		if (valid(iData.position)) {
			aTextElement.setPosition(this.convertData(iData.position,iDataTree));
		}
	
		// vertical position
		if (valid(iData.verticalposition)) {
			aTextElement.setVerticalPosition(this.convertData(iData.verticalposition,iDataTree));
		}
	
		if (valid(iData.isurl)) {
			if (iData.isurl) {
				aTextElement.isUrl();
			}
		}
	
		// translate
		if (valid(iData.translate)) {
			aTextElement.translate(this.convertData(iData.translate,iDataTree));
		}
		
		// color
		if (valid(iData.color)) {
			aTextElement.setColor(this.convertData(iData.color,iDataTree));
		}
		
		// fontsize
		if (valid(iData.linkcolor)) {
			aTextElement.setLinkColor(this.convertData(iData.linkcolor,iDataTree));
		}
		
		// color
		if (valid(iData.style)) {
			aTextElement.setStyle(this.convertData(iData.style,iDataTree));
		}
	
		if (valid(iData.onMouseOverColor)) {
			aTextElement.setMouseOverColor(this.convertData(iData.onMouseOverColor,iDataTree));
		}
		
		if (valid(iData.onMouseOverBkColor)) {
			aTextElement.setMouseOverBkColor(this.convertData(iData.onMouseOverBkColor,iDataTree));
		}
	
		return aTextElement;
	
	} else {
		logError("LayoutParser : Error no text defined for text :",iData);
		return;
	}

}

// InputTextElement Parser
LayoutParser.prototype.parseTextInputElement = function(iData,iDataTree) {

	if (valid(iData.text) && valid(iData.id)) {
	
		var aTextInputElement = new TextInputElement(this.convertData(iData.text,iDataTree),this.convertData(iData.id,iDataTree));
	
		// fontsize
		if (valid(iData.fontsize)) {
			aTextInputElement.setFontSize(this.convertData(iData.fontsize,iDataTree));
		}
	
		// position
		if (valid(iData.position)) {
			aTextInputElement.setPosition(this.convertData(iData.position,iDataTree));
		}
	
		// translate
		if (valid(iData.translate)) {
			aTextInputElement.translate(this.convertData(iData.translate,iDataTree));
		}
		
		// color
		if (valid(iData.color)) {
			aTextInputElement.setColor(this.convertData(iData.color,iDataTree));
		}
		
		// color
		if (valid(iData.padding)) {
			aTextInputElement.setPadding(this.convertData(iData.padding,iDataTree));
		}
		
		// color description
		if (valid(iData.colordesc)) {
			aTextInputElement.setColorDescription(this.convertData(iData.colordesc,iDataTree));
		}
	
		// Autocomplete function
		if (valid(iData.autocomplete)) {
			
			var auto = iData.autocomplete;
			if (valid(auto.source) && valid(auto.select) && valid(auto.min)) {
			
				aTextInputElement.setAutoComplete(
					window.recordedFunction[auto.source], 
					window.recordedFunction[auto.select],
					auto.min,
					auto.options);
			
			}
			
		}
	
		return aTextInputElement;
	
	} else {
		logError("LayoutParser : Error no text or id defined for textinput :",iData);
		return;
	}

}

// ImageElement Parser
LayoutParser.prototype.parseImageElement = function(iData,iDataTree) {

	if (valid(iData.url)) {
		
		var newURL = this.convertData(iData.url,iDataTree);
		var aImage =  new ImageElement(newURL);
		
		if (valid(iData.properties)) {
			aImage.setProperties(iData.properties);
		}
		
		if (valid(iData.onclick)) {
			aImage.onClick(window.recordedFunction[iData.onclick]);
		}
		
		return aImage;
		
	} else {
		logError("LayoutParser : Error no url element defined for image :",iData);
		return;
	}

}

// ContainerElement Parser
LayoutParser.prototype.parseContainerElement = function(iData,iDataTree) {

	if (valid(iData.inner)) {
	
		var inner = this.parse(iData.inner,iDataTree);
	
		var aContainer = new ContainerElement(inner);
	
		if (valid(iData.properties)) {
			aContainer.setProperties(iData.properties);
		}
		
		if (valid(iData.onenterbkcolor)) {
			aContainer.onEnterBkColor(iData.onenterbkcolor);
		}
		
		if (valid(iData.onclick)) {
			aContainer.onClick(window.recordedFunction[iData.onclick]);
		}
	
		if (valid(iData.style)) {
			aContainer.addStyle(this.convertData(iData.style,iDataTree));
		}
	
		return aContainer;
	
	} else {
		logError("LayoutParser : Error no inner element defined for container :",iData);
		return;
	}

}

// EmptyElement Parser
LayoutParser.prototype.parseEmptyElement = function(iData,iDataTree) {

	return new EmptyElement;

}

// LinearListLayout Parser
LayoutParser.prototype.parseLinearListLayout = function(iData,iDataTree) {

	if (valid(iData.orientation) && valid(iData.mode)) {
	
		var aLayout = new LinearListLayout(this.convertData(iData.orientation,iDataTree),this.convertData(iData.mode,iDataTree));
		
		if (valid(iData.align)) {
			aLayout.setAlignment(this.convertData(iData.align,iDataTree));
			
			if (valid(iData.verticalalign)) {
				aLayout.setAlignment(this.convertData(iData.align,iDataTree),this.convertData(iData.verticalalign,iDataTree));
			}
		}
		
		if (valid(iData.cells)) {
			
			for (var i=0; i<iData.cells.length; i++) {
			
				if (valid(iData.cellsizes)) {
					aLayout.pushElement(this.parse(iData.cells[i],iDataTree),iData.cellsizes[i]);
				} else {
					aLayout.pushElement(this.parse(iData.cells[i],iDataTree));
				}
			
			}
			
		}

		return aLayout;
	
	} else {
	
		logError("LayoutParser : Error on linear list layout definition , missing orientation or mode definition.",iData);
		return;
		
	}
	
}

LayoutParser.prototype.parseMapperElement = function(iData,iDataTree) {

	if (valid(iData.id)) {
		
		var aMapper = new MapperElement(this.convertData(iData.id,iDataTree));
		
		if (valid(iData.properties)) {
			aMapper.setProperties(iData.properties);
		}
		
		return aMapper;
	
	} else {
	
		logError("LayoutParser : Error on mapper definition , missing id .",iData);
		return;
	
	}

}

LayoutParser.prototype.parseScrollContainerElement = function(iData,iDataTree) {
	
	if (valid(iData.inner)) {
		
		var aScroll =  new ScrollContainerElement(this.parse(iData.inner,iDataTree));
	
		if (valid(iData.style)){
			aScroll.setStyle(iData.style);
		}
		
		if (valid(iData.canvasstyle)){
			aScroll.setCanvasStyle(iData.canvasstyle);
		}
	
		return aScroll;
	
	} else {

		logError("LayoutParser : Error on scroll container definition , missing id .",iData);
		return;
	
	}

}

LayoutParser.prototype.parseVerticalBarElement = function(iData,iDataTree) {

	var VBE = new VerticalBarElement();
	
	if (valid(iData.position)) {
		VBE.setPosition(iData.position);
	}
	
	if (valid(iData.style)) {
		VBE.setStyle(iData.style);
	}
	
	if (valid(iData.vstyle)) {
		VBE.setVStyle(iData.vstyle);
	}
	
	if (valid(iData.color)) {
		VBE.setColor(iData.color);
	}
	
	if (valid(iData.width)) {
		VBE.setWidth(iData.width);
	}

	if (valid(iData.height)) {
		VBE.setHeight(iData.height);
	}

	return VBE;

}

LayoutParser.prototype.parseMenuBarElement = function(iData,iDataTree) {

	var MB = new MenuBarElement();
	
	if (valid(iData.align)) {
		MB.setAlign(iData.align);
	}
	if (valid(iData.textcolor)) {
		MB.setTextColor(iData.textcolor);
	}
	if (valid(iData.fontsize)) {
		MB.setFontSize(iData.fontsize);
	}
	if (valid(iData.mouseovercolor)) {
		MB.setMouseOverColor(iData.mouseovercolor);
	}
	if (valid(iData.mouseoverbkcolor)) {
		MB.setMouseOverBkColor(iData.mouseoverbkcolor);
	}
	if (valid(iData.padding)) {
		MB.setPadding(iData.padding);
	}
	
	if (valid(iData.vstyle)) {
		MB.setVerticalBarStyle(iData.vstyle);
	}
	
	if (valid(iData.vcolor)) {
		MB.setVerticalBarColor(iData.vcolor);
	}
	
	if (valid(iData.vwidth)) {
		MB.setVerticalBarWidth(iData.vwidth);
	}
	
	if (valid(iData.vheight)) {
		MB.setVerticalBarHeight(iData.vheight);
	}

	if (valid(iData.elements)) {
		//console.log("Push elements in menu bar");
		MB.pushElements(iData.elements,iData.translates);
	}
	
	return MB;

}

LayoutParser.prototype.parseSliderElement = function(iData,iDataTree) {

	if (valid(iData.value) && valid(iData.range) && valid(iData.callback)) {
		
		var slider = new SliderElement(iData.value,iData.range,window.recordedFunction[iData.callback]);
		
		if (valid(iData.size)) {
			if (valid(iData.size.width) && valid(iData.size.height)) {
				slider.setSize(iData.size.width,iData.size.height);
			}
		}
		
		if (valid(iData.rangetype)) {
			slider.setRange(iData.range);
		}
			
		if (valid(iData.backgroundcolor)) {
			slider.setBackgroundColor(iData.backgroundcolor);
		}	
			
		return slider;

	} else {
	
		logError("LayoutParser : Error on slider definition , missing value, range, callback .",iData);
		return;
	
	}

}

// Element Getter
LayoutParser.prototype.getElement = function(iTag) {
	return this.taggedelements[iTag];
}

// Post process to build links between objects
LayoutParser.prototype.postProcess = function() {

	for (var i=0; i<this.elements.length; i++) {
	
		var aElement = this.elements[i];
		
		for (var j=0; j<this.elementTokens.length; j++) {
	
			this.postProcessFunction = this.elementPost[this.elementTokens[j]];
			
			if (valid(this.postProcessFunction)) {
			
				this.postProcessFunction(aElement);
			
			} 
		
		}
		
	
	}

}


LayoutParser.prototype.postTextInputElement = function(iElement) {
	
	if (valid(iElement.content.follow)) { //optionnal
	
		var aElementToFollow = this.getElement(iElement.content.follow);
		if (valid(aElementToFollow)) {
		
			iElement.element.setFollowing(aElementToFollow.element);
		
		} else {
			logError("LayoutParser : Error on post process, tagged element \""+iElement.content.follow+"\" not defined.",iElement);
			return;
		}
	
	}

}


/* Layout Manager JBV 2012-2013 */



var LayoutManager = function(iInsertPoint,iStartEvent){

	this._fsmStates={};
	this._layoutMap={};
	this._insertPoint=iInsertPoint;
	this._startEvent = iStartEvent; // Check start point is a state !!!
	this._hasStarted = false;
	HashCall.pushHashFunction(LayoutManager.hashcall(this));
} 

LayoutManager.prototype.start = function(){
	
	this._hasStarted = true;
	var aState = this.resolveNextState(this._startEvent)
	this.startPoint(aState.layoutname);
	
	var aSplit = getLocationSplit();
	aSplit.ref = this._startEvent;
	var aHash = buildLocationHash(aSplit);
	TextElement.hashcall(aHash); // this is to force the links
									  				 // to be loaded after

	$.history.load(aHash);
	
}

LayoutManager.prototype.setFailsafe = function(iFunction){
	this._failSafeFunction = iFunction;
}

// Init calls for hash management

LayoutManager.hashcall = function(iManager){

	return function(iHash) {
		
		var aManager = iManager;
		//console.log("Manager to be called:"+iHash);
		//console.log(aManager);
	
		var aEvent =  getHashSplit(iHash);
		aManager.callEvent(aEvent.ref, iHash);
		
		//TextElement.hashcall(iHash); // this is to force the links
									 // to be loaded after
	
	}
	
}


LayoutManager.prototype.addLayout = function(iLayoutName,iLayout,iDialog) {

	this._layoutMap[iLayoutName] = {};
	this._layoutMap[iLayoutName].layout = iLayout;
	this._layoutMap[iLayoutName].inserted = false;
	this._layoutMap[iLayoutName].dialog = iDialog;
	
}

LayoutManager.prototype.addFSMEvent = function(iLayoutName1, iLayoutName2, iEvent, iFunction){

	var aLayoutName1 = iLayoutName1;
	if (aLayoutName1 == "*" ) {
		aLayoutName1 = "_STAR_";
	}

	if (typeof this._fsmStates[iEvent]==="undefined") {
		this._fsmStates[iEvent] ={}
		this._fsmStates[iEvent][aLayoutName1] = {};
		this._fsmStates[iEvent][aLayoutName1].layoutname = iLayoutName2;
		if (valid(iFunction)) {this._fsmStates[iEvent][aLayoutName1].callFunction = iFunction;}
	} else {
		if (typeof this._fsmStates[iEvent][aLayoutName1]==="undefined") {
			this._fsmStates[iEvent][aLayoutName1] = {};
			this._fsmStates[iEvent][aLayoutName1].layoutname = iLayoutName2;
			if (valid(iFunction)) {this._fsmStates[iEvent][aLayoutName1].callFunction = iFunction;}
		} else {
			alert("FSM Error : state already defined ("+aLayoutName1+","+iLayoutName2+","+iEvent+")");
		}
	}

}

LayoutManager.prototype.hideCurrent = function(){
	$("#"+this._start).fadeOut('slow');
}

LayoutManager.prototype.forceLayoutResize = function(){

	console.log("Force Layout Resize");
	forceResize = true;
	console.log(this._start);
	handleDependentResize($("#"+this._start),"",true);
	
}

LayoutManager.prototype.showCurrent = function(){
	$("#"+this._start).fadeIn('slow').promise().done(this.forceLayoutResize());
}

LayoutManager.prototype.startPoint = function(iLayoutName){
	
	//console.log("Start point:"+iLayoutName);
	
	var prev = this._start;
	this._start = iLayoutName;
	
	if (typeof this._layoutMap[this._start]==="undefined") {
		console.log("FSM Error : start point not defined ("+this._start+")");
		if (valid(this._failSafeFunction)) {
				this.callFailsafe();
		} 
		return;
	}
	
	if (this._layoutMap[this._start].inserted) {
		
		this.shiftLayout(this._start, true, +1000);
		
	} else {
		
		//console.log("Insert the layout in the DOM");
		this._layoutMap[this._start].inserted = true;
		
		//console.log("Insert layout in the DOM:");
		//console.log($(this._insertPoint));
		$(this._insertPoint).append("<div id='"+iLayoutName+"' style='position:absolute; top 0px; left: 0px; margin : 0px; padding: 0px; overflow : hidden; width:100%; height:100%;' ></div>");
		
		var aLayout = this._layoutMap[this._start].layout;
		aLayout.insert("#"+iLayoutName);
		
	}
	
	
	
}

LayoutManager.prototype.shiftLayout = function(iName,iShow,iValue){

	var aLayout = this._layoutMap[iName].layout;
	
	if (iShow) {
		$("#"+iName).css("visibility","visible");
		aLayout.show().shiftZIndex(iValue);
	} else {
		$("#"+iName).css("visibility","hidden");
		aLayout.hide().shiftZIndex(iValue);
	}
	
	var currentZIndex = $("#"+iName).css("z-index");
	$("#"+iName).css( 'z-index' , currentZIndex+iValue );

}

LayoutManager.prototype.pushEvent = function(iEvent){
	var aSplit = {}; //getLocationSplit(); // reset context
	aSplit.ref = iEvent;
	$.history.load(buildLocationHash(aSplit));
}

LayoutManager.prototype.callFailsafe = function() {
	this._failSafeFunction();
}

LayoutManager.prototype.resolveNextState =  function(iEvent) {

	var aEventMap = this._fsmStates[iEvent];
	if (typeof aEventMap==="undefined") {
		console.log("FSM Error : event not defined ("+iEvent+")");
		if (valid(this._failSafeFunction)) {
				this.callFailsafe();
		} 
		return;
	}
	
	var aFromState = aEventMap[this._start];
	if (typeof aFromState==="undefined") {
	
		aFromState = aEventMap["_STAR_"];
		
		if (!valid(aFromState)) {
			console.log("FSM Error : transition not defined for this event ("+this._start+","+iEvent+")");
			
			if (valid(this._failSafeFunction)) {
				this.callFailsafe();
			} 
			
			return;
		}
		
	}

	return aFromState;
}

LayoutManager.prototype.callEvent = function(iEvent,iHash){
	
	if (this._hasStarted) {
	
		var aToState = this.resolveNextState(iEvent);
	
		if (valid(aToState)) { // else failsafe has been called

			this.shiftLayout(this._start, false, -1000);
			this.startPoint(aToState.layoutname);
			
			//console.log("Force TextElement");
			if (valid(iHash)) {
				//console.log("To hash:"+iHash);
				TextElement.hashcall(iHash); // this is to force the links
										     // to be loaded after
			}
		
			if (valid(aToState.callFunction)) {
				aToState.callFunction();
			} 
		
		}
	
	} else {
	
		this._startEvent = iEvent;
	
	}
	
}
// ######################## EVENT HANDLER ON WINDOW RESIZE #############

var aInitHashmapResize = [];

function handleDependentResize(iElement,iChain,iForceResize) {

	//console.log('handleDependentResize');
	//console.log(iElement);
	//console.log('selected elements:');

	var aElement = $(iElement);

	$.each( aElement.nearest(ELEMENTCLASSLIST),
			function(key,value) {
			
				// TODO to be changed and use POO ? or hash table 
				//console.log(value);
				//console.log($(value));
				var aClass = $(value).attr("class").split(" ");
				//console.log("Class:#"+aClass+"#");
				if (aClass.length>1) {
					for (var i=0;i<aClass.length;i++) {
						var aObject = aInitHashmapResize[aClass[i]];
						if (valid(aObject)) {
							Element.resizeInChain(aObject,value,iChain,iForceResize);
						}
					}
				} else {
					var aObject = aInitHashmapResize[aClass[0]];
					Element.resizeInChain(aObject,value,iChain,iForceResize);
				}
				
			
			}
	);
	
}

function handleLayoutResize() {

	//console.log("Handle layout resize:");
	handleDependentResize($(document));
		
}

function initHashTableResize() {

	aInitHashmapResize["gridlayout"] = GridLayout;
	aInitHashmapResize["containerElement"] = ContainerElement;
	aInitHashmapResize["scrollContainerElement"] = ScrollContainerElement;
	aInitHashmapResize["textelement"] = TextElement;
	aInitHashmapResize["textinputelement"] = TextInputElement;
	aInitHashmapResize["imgelement"] = ImageElement;
	aInitHashmapResize["linearlistlayout"] = LinearListLayout;
	aInitHashmapResize["mapperelement"] = MapperElement;
	aInitHashmapResize["verticalbarelement"] = VerticalBarElement;
	aInitHashmapResize["sliderelement"] = SliderElement;
	aInitHashmapResize["buttonelement"] =  ButtonElement;

}

initHashTableResize();
hasToBeResized(handleLayoutResize);